@@ -49,14 +49,17 @@
 #include "Selection.h"
 #include "PositionCache.h"
 #include "Editor.h"
-//#include "ScintillaCallTip.h"
 
 #include "ScintillaBase.h"
 
 extern "C" NSString* ScintillaRecPboardType;
 
+@class InnerView;
+@class MarginView;
 @class ScintillaView;
 
+@class FindHighlightLayer;
+
 /**
  * Helper class to be used as timer target (NSTimer).
  */
@@ -107,6 +110,8 @@ private:
 
   bool capturedMouse;
 
+  bool enteredSetScrollingSize;
+
   // Private so ScintillaCocoa objects can not be copied
   ScintillaCocoa(const ScintillaCocoa &) : ScintillaBase() {}
   ScintillaCocoa &operator=(const ScintillaCocoa &) { return * this; }
@@ -118,8 +123,12 @@ private:
   int scrollTicks;
   NSTimer* tickTimer;
   NSTimer* idleTimer;
+  CFRunLoopObserverRef observer;
 	
+  FindHighlightLayer *layerFindIndicator;
+
 protected:
+  Point GetVisibleOriginInMain();
   PRectangle GetClientRectangle();
   Point ConvertPoint(NSPoint point);
   
@@ -127,19 +136,22 @@ protected:
   virtual void Finalise();
   virtual CaseFolder *CaseFolderForEncoding();
   virtual std::string CaseMapString(const std::string &s, int caseMapping);
-public:
-  NSView* ContentView();
+  virtual void CancelModes();
 
-  ScintillaCocoa(NSView* view);
+public:
+  ScintillaCocoa(InnerView* view, MarginView* viewMargin);
   virtual ~ScintillaCocoa();
 
   void RegisterNotifyCallback(intptr_t windowid, SciNotifyFunc callback);
   sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
 
   ScintillaView* TopContainer();
+  NSScrollView* ScrollContainer();
+  InnerView* ContentView();
 
-  void SyncPaint(void* gc, PRectangle rc);
-  void Draw(NSRect rect, CGContextRef gc);
+  bool SyncPaint(void* gc, PRectangle rc);
+  bool Draw(NSRect rect, CGContextRef gc);
+  void PaintMargin(NSRect aRect);
 
   virtual sptr_t DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
   void SetTicking(bool on);
@@ -150,9 +162,10 @@ public:
   void SetVerticalScrollPos();
   void SetHorizontalScrollPos();
   bool ModifyScrollBars(int nMax, int nPage);
+  bool SetScrollingSize(void);
   void Resize();
-  void DoScroll(float position, NSScrollerPart part, bool horizontal);
-    
+  void UpdateForScroll();
+
   // Notifications for the owner.
   void NotifyChange();
   void NotifyFocus(bool focus);
@@ -179,8 +192,14 @@ public:
 
   void TimerFired(NSTimer* timer);
   void IdleTimerFired();
+  static void UpdateObserver(CFRunLoopObserverRef observer, CFRunLoopActivity activity, void *sci);
+  void ObserverAdd();
+  void ObserverRemove();
+  virtual void IdleWork();
+  virtual void QueueIdleWork(WorkNeeded::workItems items, int upTo);
   int InsertText(NSString* input);
-  
+  void SelectOnlyMainSelection();
+
   bool KeyboardInput(NSEvent* event);
   void MouseDown(NSEvent* event);
   void MouseMove(NSEvent* event);
@@ -209,6 +228,11 @@ public:
   void HandleCommand(NSInteger command);
 
   virtual void ActiveStateChanged(bool isActive);
+
+  // Find indicator
+  void ShowFindIndicatorForRange(NSRange charRange, BOOL retaining);
+  void MoveFindIndicatorWithBounce(BOOL bounce);
+  void HideFindIndicator();
 };
 
 
