@@ -15,8 +15,11 @@
  */
 
 #import <Cocoa/Cocoa.h>
-
-#import <Carbon/Carbon.h> // Temporary
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+#import <QuartzCore/CAGradientLayer.h>
+#endif
+#import <QuartzCore/CAAnimation.h>
+#import <QuartzCore/CATransaction.h>
 
 #include "ScintillaView.h"
 #include "PlatCocoa.h"
@@ -107,7 +110,7 @@ static const KeyToCommand macMapDefault[] =
   {SCK_BACK,      SCI_NORM,   SCI_DELETEBACK},
   {SCK_BACK,      SCI_SHIFT,  SCI_DELETEBACK},
   {SCK_BACK,      SCI_CTRL,   SCI_DELWORDLEFT},
-  {SCK_BACK,      SCI_ALT,    SCI_UNDO},
+  {SCK_BACK,      SCI_ALT,    SCI_DELWORDLEFT},
   {SCK_BACK,      SCI_CSHIFT, SCI_DELLINELEFT},
   {'z',           SCI_CMD,    SCI_UNDO},
   {'z',           SCI_SCMD,   SCI_REDO},
@@ -134,6 +137,174 @@ static const KeyToCommand macMapDefault[] =
 
 //--------------------------------------------------------------------------------------------------
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+
+// Only implement FindHighlightLayer on OS X 10.6+
+
+/**
+ * Class to display the animated gold roundrect used on OS X for matches.
+ */
+@interface FindHighlightLayer : CAGradientLayer
+{
+@private
+	NSString *sFind;
+	int positionFind;
+	BOOL retaining;
+	CGFloat widthText;
+	CGFloat heightLine;
+	NSString *sFont;
+	CGFloat fontSize;
+}
+
+@property (copy) NSString *sFind;
+@property (assign) int positionFind;
+@property (assign) BOOL retaining;
+@property (assign) CGFloat widthText;
+@property (assign) CGFloat heightLine;
+@property (copy) NSString *sFont;
+@property (assign) CGFloat fontSize;
+
+- (void) animateMatch: (CGPoint)ptText bounce:(BOOL)bounce;
+- (void) hideMatch;
+
+@end
+
+//--------------------------------------------------------------------------------------------------
+
+@implementation FindHighlightLayer
+
+@synthesize sFind, positionFind, retaining, widthText, heightLine, sFont, fontSize;
+
+-(id) init {
+	if (self = [super init]) {
+		[self setNeedsDisplayOnBoundsChange: YES];
+		// A gold to slightly redder gradient to match other applications
+		CGColorRef colGold = CGColorCreateGenericRGB(1.0, 1.0, 0, 1.0);
+		CGColorRef colGoldRed = CGColorCreateGenericRGB(1.0, 0.8, 0, 1.0);
+		self.colors = [NSArray arrayWithObjects:(id)colGoldRed, (id)colGold, nil];
+		CGColorRelease(colGoldRed);
+		CGColorRelease(colGold);
+
+		CGColorRef colGreyBorder = CGColorCreateGenericGray(0.756f, 0.5f);
+		self.borderColor = colGreyBorder;
+		CGColorRelease(colGreyBorder);
+
+		self.borderWidth = 1.0;
+		self.cornerRadius = 5.0f;
+		self.shadowRadius = 1.0f;
+		self.shadowOpacity = 0.9f;
+		self.shadowOffset = CGSizeMake(0.0f, -2.0f);
+		self.anchorPoint = CGPointMake(0.5, 0.5);
+	}
+	return self;
+	
+}
+
+const CGFloat paddingHighlightX = 4;
+const CGFloat paddingHighlightY = 2;
+
+-(void) drawInContext:(CGContextRef)context {
+	if (!sFind || !sFont)
+		return;
+	
+	CFStringRef str = CFStringRef(sFind);
+	
+	CFMutableDictionaryRef styleDict = CFDictionaryCreateMutable(kCFAllocatorDefault, 2,
+								     &kCFTypeDictionaryKeyCallBacks, 
+								     &kCFTypeDictionaryValueCallBacks);
+	CGColorRef color = CGColorCreateGenericRGB(0.0, 0.0, 0.0, 1.0);
+	CFDictionarySetValue(styleDict, kCTForegroundColorAttributeName, color);
+	CTFontRef fontRef = ::CTFontCreateWithName((CFStringRef)sFont, fontSize, NULL);
+	CFDictionaryAddValue(styleDict, kCTFontAttributeName, fontRef);
+	
+	CFAttributedStringRef attrString = ::CFAttributedStringCreate(NULL, str, styleDict);
+	CTLineRef textLine = ::CTLineCreateWithAttributedString(attrString);
+	// Indent from corner of bounds
+	CGContextSetTextPosition(context, paddingHighlightX, 3 + paddingHighlightY);
+	CTLineDraw(textLine, context);
+	
+	CFRelease(textLine);
+	CFRelease(attrString);
+	CFRelease(fontRef);
+	CGColorRelease(color);
+	CFRelease(styleDict);
+}
+
+- (void) animateMatch: (CGPoint)ptText bounce:(BOOL)bounce {
+	if (!self.sFind || ![self.sFind length]) {
+		[self hideMatch];
+		return;
+	}
+
+	CGFloat width = self.widthText + paddingHighlightX * 2;
+	CGFloat height = self.heightLine + paddingHighlightY * 2;
+
+	CGFloat flipper = self.geometryFlipped ? -1.0 : 1.0;
+
+	// Adjust for padding
+	ptText.x -= paddingHighlightX;
+	ptText.y += flipper * paddingHighlightY;
+
+	// Shift point to centre as expanding about centre
+	ptText.x += width / 2.0;
+	ptText.y -= flipper * height / 2.0;
+
+	[CATransaction begin];
+	[CATransaction setValue:[NSNumber numberWithFloat:0.0] forKey:kCATransactionAnimationDuration];
+	self.bounds = CGRectMake(0,0, width, height);
+	self.position = ptText;
+	if (bounce) {
+		// Do not reset visibility when just moving
+		self.hidden = NO;
+		self.opacity = 1.0;
+	}
+	[self setNeedsDisplay];
+	[CATransaction commit];
+	
+	if (bounce) {
+		CABasicAnimation *animBounce = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
+		animBounce.duration = 0.15;
+		animBounce.autoreverses = YES;
+		animBounce.removedOnCompletion = NO;
+		animBounce.fromValue = [NSNumber numberWithFloat: 1.0];
+		animBounce.toValue = [NSNumber numberWithFloat: 1.25];
+		
+		if (self.retaining) {
+			
+			[self addAnimation: animBounce forKey:@"animateFound"];
+			
+		} else {
+			
+			CABasicAnimation *animFade = [CABasicAnimation animationWithKeyPath:@"opacity"];
+			animFade.duration = 0.1;
+			animFade.beginTime = 0.4;
+			animFade.removedOnCompletion = NO;
+			animFade.fromValue = [NSNumber numberWithFloat: 1.0];
+			animFade.toValue = [NSNumber numberWithFloat: 0.0];
+			
+			CAAnimationGroup *group = [CAAnimationGroup animation];
+			[group setDuration:0.5];
+			group.removedOnCompletion = NO;
+			group.fillMode = kCAFillModeForwards;
+			[group setAnimations:[NSArray arrayWithObjects:animBounce, animFade, nil]];
+			
+			[self addAnimation:group forKey:@"animateFound"];
+		}
+	}
+}
+
+- (void) hideMatch {
+	self.sFind = @"";
+	self.positionFind = INVALID_POSITION;
+	self.hidden = YES;
+}
+
+@end
+
+#endif
+
+//--------------------------------------------------------------------------------------------------
+
 @implementation TimerTarget
 
 - (id) init: (void*) target
@@ -156,6 +327,8 @@ static const KeyToCommand macMapDefault[] =
 
 - (void) dealloc
 {
+  NSNotificationCenter* center = [NSNotificationCenter defaultCenter];
+  [center removeObserver:self];
   [notificationQueue release];
   [super dealloc];
 }
@@ -206,10 +379,23 @@ static const KeyToCommand macMapDefault[] =
 
 //----------------- ScintillaCocoa -----------------------------------------------------------------
 
-ScintillaCocoa::ScintillaCocoa(NSView* view)
+ScintillaCocoa::ScintillaCocoa(InnerView* view, MarginView* viewMargin)
 {
-  wMain= [view retain];
-  timerTarget = [[[TimerTarget alloc] init: this] retain];
+  vs.marginInside = false;
+  wMain = view; // Don't retain since we're owned by view, which would cause a cycle
+  wMargin = viewMargin;
+  timerTarget = [[TimerTarget alloc] init: this];
+  lastMouseEvent = NULL;
+  notifyObj = NULL;
+  notifyProc = NULL;
+  capturedMouse = false;
+  enteredSetScrollingSize = false;
+  scrollSpeed = 1;
+  scrollTicks = 2000;
+  tickTimer = NULL;
+  idleTimer = NULL;
+  observer = NULL;
+  layerFindIndicator = NULL;
   Initialise();
 }
 
@@ -219,8 +405,6 @@ ScintillaCocoa::~ScintillaCocoa()
 {
   SetTicking(false);
   [timerTarget release];
-  NSView* container = ContentView();
-  [container release];
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -230,16 +414,7 @@ ScintillaCocoa::~ScintillaCocoa()
  */
 void ScintillaCocoa::Initialise() 
 {
-  static bool initedLexers = false;
-  if (!initedLexers)
-  {
-    initedLexers = true;
-    Scintilla_LinkLexers();
-  }
-  notifyObj = NULL;
-  notifyProc = NULL;
-  
-  capturedMouse = false;
+  Scintilla_LinkLexers();
   
   // Tell Scintilla not to buffer: Quartz buffers drawing for us.
   WndProc(SCI_SETBUFFEREDDRAW, 0, 0);
@@ -260,12 +435,70 @@ void ScintillaCocoa::Initialise()
  */
 void ScintillaCocoa::Finalise()
 {
+  ObserverRemove();
   SetTicking(false);
   ScintillaBase::Finalise();
 }
 
 //--------------------------------------------------------------------------------------------------
 
+void ScintillaCocoa::UpdateObserver(CFRunLoopObserverRef observer, CFRunLoopActivity activity, void *info) {
+  ScintillaCocoa* sci = reinterpret_cast<ScintillaCocoa*>(info);
+  sci->IdleWork();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Add an observer to the run loop to perform styling as high-priority idle task.
+ */
+
+void ScintillaCocoa::ObserverAdd() {
+  if (!observer) {
+    CFRunLoopObserverContext context;
+    context.version = 0;
+    context.info = this;
+    context.retain = NULL;
+    context.release = NULL;
+    context.copyDescription = NULL;
+
+    CFRunLoopRef mainRunLoop = CFRunLoopGetMain();
+    observer = CFRunLoopObserverCreate(NULL, kCFRunLoopEntry | kCFRunLoopBeforeWaiting,
+      true, 0, UpdateObserver, &context);
+    CFRunLoopAddObserver(mainRunLoop, observer, kCFRunLoopCommonModes);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Remove the run loop observer.
+ */
+void ScintillaCocoa::ObserverRemove() {
+  if (observer) {
+    CFRunLoopRef mainRunLoop = CFRunLoopGetMain();
+    CFRunLoopRemoveObserver(mainRunLoop, observer, kCFRunLoopCommonModes);
+    CFRelease(observer);
+  }
+  observer = NULL;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ScintillaCocoa::IdleWork() {
+  Editor::IdleWork();
+  ObserverRemove();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void ScintillaCocoa::QueueIdleWork(WorkNeeded::workItems items, int upTo) {
+  Editor::QueueIdleWork(items, upTo);
+  ObserverAdd();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Convert a core foundation string into an array of bytes in a particular encoding
  */
@@ -307,13 +540,14 @@ public:
                                                             locale:[NSLocale currentLocale]];
 
             const char *cpMapped = [sMapped UTF8String];
-			size_t lenMapped = strlen(cpMapped);
+			size_t lenMapped = cpMapped ? strlen(cpMapped) : 0;
 			if (lenMapped < sizeFolded) {
 				memcpy(folded, cpMapped,  lenMapped);
 			} else {
 				lenMapped = 0;
 			}
-            CFRelease(cfsVal);
+			if (cfsVal)
+				CFRelease(cfsVal);
 			return lenMapped;
 		}
 	}
@@ -430,13 +664,33 @@ std::string ScintillaCocoa::CaseMapString(const std::string &s, int caseMapping)
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Cancel all modes, both for base class and any find indicator.
+ */
+void ScintillaCocoa::CancelModes() {
+  ScintillaBase::CancelModes();
+  HideFindIndicator();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Helper function to get the outer container which represents the Scintilla editor on application side.
  */
 ScintillaView* ScintillaCocoa::TopContainer()
 {
   NSView* container = static_cast<NSView*>(wMain.GetID());
-  return static_cast<ScintillaView*>([container superview]);
+  return static_cast<ScintillaView*>([[[container superview] superview] superview]);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Helper function to get the scrolling view.
+ */
+NSScrollView* ScintillaCocoa::ScrollContainer() {
+  NSView* container = static_cast<NSView*>(wMain.GetID());
+  return static_cast<NSScrollView*>([[container superview] superview]);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -444,9 +698,21 @@ ScintillaView* ScintillaCocoa::TopContainer()
 /**
  * Helper function to get the inner container which represents the actual "canvas" we work with.
  */
-NSView* ScintillaCocoa::ContentView()
+InnerView* ScintillaCocoa::ContentView()
 {
-  return static_cast<NSView*>(wMain.GetID());
+  return static_cast<InnerView*>(wMain.GetID());
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Return the top left visible point relative to the origin point of the whole document.
+ */
+Scintilla::Point ScintillaCocoa::GetVisibleOriginInMain()
+{
+  NSScrollView *scrollView = ScrollContainer();
+  NSRect contentRect = [[scrollView contentView] bounds];
+  return Point(contentRect.origin.x, contentRect.origin.y);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -454,26 +720,29 @@ NSView* ScintillaCocoa::ContentView()
 /**
  * Instead of returning the size of the inner view we have to return the visible part of it
  * in order to make scrolling working properly.
+ * The returned value is in document coordinates.
  */
 PRectangle ScintillaCocoa::GetClientRectangle()
 {
-  NSView* host = ContentView();
-  NSSize size = [host frame].size;
-  return PRectangle(0, 0, size.width, size.height);
+  NSScrollView *scrollView = ScrollContainer();
+  NSSize size = [[scrollView contentView] bounds].size;
+  Point origin = GetVisibleOriginInMain();
+  return PRectangle(origin.x, origin.y, origin.x+size.width, origin.y + size.height);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
  * Converts the given point from base coordinates to local coordinates and at the same time into
- * a native Point structure. Base coordinates are used for the top window used in the view hierarchy. 
+ * a native Point structure. Base coordinates are used for the top window used in the view hierarchy.
+ * Returned value is in view coordinates. 
  */
 Scintilla::Point ScintillaCocoa::ConvertPoint(NSPoint point)
 {
   NSView* container = ContentView();
   NSPoint result = [container convertPoint: point fromView: nil];
-  
-  return Point(result.x, result.y);
+  Scintilla::Point ptOrigin = GetVisibleOriginInMain();
+  return Point(result.x - ptOrigin.x, result.y - ptOrigin.y);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -510,10 +779,10 @@ sptr_t scintilla_send_message(void* sci, unsigned int iMessage, uptr_t wParam, s
 
 //--------------------------------------------------------------------------------------------------
 
-/** 
+/**
  * That's our fake window procedure. On Windows each window has a dedicated procedure to handle
  * commands (also used to synchronize UI and background threads), which is not the case in Cocoa.
- * 
+ *
  * Messages handled here are almost solely for special commands of the backend. Everything which
  * would be sytem messages on Windows (e.g. for key down, mouse move etc.) are handled by
  * directly calling appropriate handlers.
@@ -529,15 +798,27 @@ sptr_t ScintillaCocoa::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lPar
       return reinterpret_cast<sptr_t>(this);
       
     case SCI_GRABFOCUS:
-	  [[ContentView() window] makeFirstResponder:ContentView()];
+      [[ContentView() window] makeFirstResponder:ContentView()];
       break;
-
+      
     case SCI_SETBUFFEREDDRAW:
-      // Buffered drawing not supported on Cocoa 
+      // Buffered drawing not supported on Cocoa
       bufferedDraw = false;
       break;
-
-    case WM_UNICHAR: 
+      
+    case SCI_FINDINDICATORSHOW:
+      ShowFindIndicatorForRange(NSMakeRange(wParam, lParam-wParam), YES);
+      return 0;
+      
+    case SCI_FINDINDICATORFLASH:
+      ShowFindIndicatorForRange(NSMakeRange(wParam, lParam-wParam), NO);
+      return 0;
+      
+    case SCI_FINDINDICATORHIDE:
+      HideFindIndicator();
+      return 0;
+      
+    case WM_UNICHAR:
       // Special case not used normally. Characters passed in this way will be inserted
       // regardless of their value or modifier states. That means no command interpretation is
       // performed.
@@ -1154,10 +1435,10 @@ void ScintillaCocoa::SetPasteboardData(NSPasteboard* board, const SelectionText
                                                reinterpret_cast<const UInt8 *>(selectedText.s), 
                                                selectedText.len-1, encoding, false);
 
-  [board declareTypes:[NSArray arrayWithObjects:
-                       NSStringPboardType,
-                       selectedText.rectangular ? ScintillaRecPboardType : nil,
-                       nil] owner:nil];
+  NSArray *pbTypes = selectedText.rectangular ?
+    [NSArray arrayWithObjects: NSStringPboardType, ScintillaRecPboardType, nil] :
+    [NSArray arrayWithObjects: NSStringPboardType, nil];
+  [board declareTypes:pbTypes owner:nil];
   
   if (selectedText.rectangular)
   {
@@ -1167,7 +1448,8 @@ void ScintillaCocoa::SetPasteboardData(NSPasteboard* board, const SelectionText
   
   [board setString: (NSString *)cfsVal forType: NSStringPboardType];
 
-  CFRelease(cfsVal);
+  if (cfsVal)
+    CFRelease(cfsVal);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1189,7 +1471,7 @@ bool ScintillaCocoa::GetPasteboardData(NSPasteboard* board, SelectionText* selec
     {
       CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
                                                            vs.styles[STYLE_DEFAULT].characterSet);
-      CFRange rangeAll = {0, [data length]};
+      CFRange rangeAll = {0, static_cast<CFIndex>([data length])};
       CFIndex usedLen = 0;
       CFStringGetBytes((CFStringRef)data, rangeAll, encoding, '?',
                        false, NULL, 0, &usedLen);
@@ -1219,16 +1501,6 @@ bool ScintillaCocoa::GetPasteboardData(NSPasteboard* board, SelectionText* selec
 void ScintillaCocoa::SetMouseCapture(bool on)
 {
   capturedMouse = on;
-  /*
-  if (mouseDownCaptures)
-  {
-    if (capturedMouse)
-      WndProc(SCI_SETCURSOR, Window::cursorArrow, 0);
-    else
-      // Reset to normal. Actual image will be set on mouse move.
-      WndProc(SCI_SETCURSOR, (unsigned int) SC_CURSORNORMAL, 0);
-  }
-   */
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1243,108 +1515,106 @@ bool ScintillaCocoa::HaveMouseCapture()
 /**
  * Synchronously paint a rectangle of the window.
  */
-void ScintillaCocoa::SyncPaint(void* gc, PRectangle rc)
+bool ScintillaCocoa::SyncPaint(void* gc, PRectangle rc)
 {
   paintState = painting;
   rcPaint = rc;
   PRectangle rcText = GetTextRectangle();
   paintingAllText = rcPaint.Contains(rcText);
+  bool succeeded = true;
   Surface *sw = Surface::Allocate(SC_TECHNOLOGY_DEFAULT);
   if (sw)
   {
     sw->Init(gc, wMain.GetID());
     Paint(sw, rc);
-    if (paintState == paintAbandoned)
-    {
-      // Do a full paint.
-      rcPaint = GetClientRectangle();
-      paintState = painting;
-      paintingAllText = true;
-      Paint(sw, rcPaint);
-    }
+    succeeded = paintState != paintAbandoned;
     sw->Release();
     delete sw;
   }
   paintState = notPainting;
+  if (!succeeded)
+  {
+    NSView *marginView = static_cast<NSView*>(wMargin.GetID());
+    [marginView setNeedsDisplay:YES];
+  }
+  return succeeded;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Scrolls the pixels in the window some number of lines.
- * Invalidates the pixels scrolled into view.
+ * Paint the margin into the MarginView space.
+ */
+void ScintillaCocoa::PaintMargin(NSRect aRect)
+{
+  CGContextRef gc = (CGContextRef) [[NSGraphicsContext currentContext] graphicsPort];
+
+  PRectangle rc = NSRectToPRectangle(aRect);
+  rcPaint = rc;
+  Surface *sw = Surface::Allocate(SC_TECHNOLOGY_DEFAULT);
+  if (sw)
+  {
+    sw->Init(gc, wMargin.GetID());
+    PaintSelMargin(sw, rc);
+    sw->Release();
+    delete sw;
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * ScrollText is empty because scrolling is handled by the NSScrollView.
  */
 void ScintillaCocoa::ScrollText(int linesToMove)
 {
-	// Move those pixels
-	NSView *content = ContentView();
-    
-	[content lockFocus];
-	int diff = vs.lineHeight * linesToMove;
-	PRectangle textRect = GetTextRectangle();
-	// Include margins as they must scroll
-	textRect.left = 0;
-	NSRect textRectangle = PRectangleToNSRect(textRect);
-	NSPoint destPoint = textRectangle.origin;
-    destPoint.y += diff;
-	NSCopyBits(0, textRectangle, destPoint);
-
-	// Paint them nice
-	NSRect redrawRectangle = textRectangle;
-	if (linesToMove < 0) {
-		// Repaint bottom
-		redrawRectangle.origin.y = redrawRectangle.origin.y + redrawRectangle.size.height + diff;
-		redrawRectangle.size.height = -diff;
-	} else {
-		// Repaint top
-		redrawRectangle.size.height = diff;
-	}
-	
-	[content drawRect: redrawRectangle];
-	[content unlockFocus];
-
-	// If no flush done here then multiple scrolls will get buffered and screen 
-	// will only update a few times a second.
-	//[[content window] flushWindow];
-    // However, doing the flush leads to the caret updating as a separate operation
-    // which looks bad when scrolling by holding down the down arrow key.
-
-	// Could invalidate instead of synchronous draw but that may not be as smooth
-	//[content setNeedsDisplayInRect: redrawRectangle];
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Modfies the vertical scroll position to make the current top line show up as such.
+ * Modifies the vertical scroll position to make the current top line show up as such.
  */
 void ScintillaCocoa::SetVerticalScrollPos()
 {
-  ScintillaView* topContainer = TopContainer();
-  
-  // Convert absolute coordinate into the range [0..1]. Keep in mind that the visible area
-  // does *not* belong to the scroll range.
-  float relativePosition = (float) topLine / MaxScrollPos();
-  [topContainer setVerticalScrollPosition: relativePosition];
+  NSScrollView *scrollView = ScrollContainer();
+  if (scrollView) {
+    NSClipView *clipView = [scrollView contentView];
+    NSRect contentRect = [clipView bounds];
+    [clipView scrollToPoint: NSMakePoint(contentRect.origin.x, topLine * vs.lineHeight)];
+    [scrollView reflectScrolledClipView:clipView];
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Modifies the horizontal scroll position to match xOffset.
+ */
 void ScintillaCocoa::SetHorizontalScrollPos()
 {
-  ScintillaView* topContainer = TopContainer();
   PRectangle textRect = GetTextRectangle();
-  
-  // Convert absolute coordinate into the range [0..1]. Keep in mind that the visible area
-  // does *not* belong to the scroll range.
-  float relativePosition = (float) xOffset / (scrollWidth - textRect.Width());
-  [topContainer setHorizontalScrollPosition: relativePosition];
+
+  int maxXOffset = scrollWidth - textRect.Width();
+  if (maxXOffset < 0)
+    maxXOffset = 0;
+  if (xOffset > maxXOffset)
+    xOffset = maxXOffset;
+  NSScrollView *scrollView = ScrollContainer();
+  if (scrollView) {
+    NSClipView * clipView = [scrollView contentView];
+    NSRect contentRect = [clipView bounds];
+    [clipView scrollToPoint: NSMakePoint(xOffset, contentRect.origin.y)];
+    [scrollView reflectScrolledClipView:clipView];
+  }
+  MoveFindIndicatorWithBounce(NO);
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
  * Used to adjust both scrollers to reflect the current scroll range and position in the editor.
+ * Arguments no longer used as NSScrollView handles details of scroll bar sizes.
  *
  * @param nMax Number of lines in the editor.
  * @param nPage Number of lines per scroll page.
@@ -1352,103 +1622,66 @@ void ScintillaCocoa::SetHorizontalScrollPos()
  */
 bool ScintillaCocoa::ModifyScrollBars(int nMax, int nPage)
 {
-#pragma unused(nPage)
-  // Input values are given in lines, not pixels, so we have to convert.
-  int lineHeight = static_cast<int>(WndProc(SCI_TEXTHEIGHT, 0, 0));
-  PRectangle bounds = GetTextRectangle();
-  ScintillaView* topContainer = TopContainer();
+#pragma unused(nMax, nPage)
+  return SetScrollingSize();
+}
 
-  // Set page size to the same value as the scroll range to hide the scrollbar.
-  int scrollRange = lineHeight * (nMax + 1); // +1 because the caller subtracted one.
-  int pageSize;
-  if (verticalScrollBarVisible)
-    pageSize = bounds.Height();
-  else
-    pageSize = scrollRange;
-  bool verticalChange = [topContainer setVerticalScrollRange: scrollRange page: pageSize];
-  
-  scrollRange = scrollWidth;
-  if (horizontalScrollBarVisible)
-    pageSize = bounds.Width();
-  else
-    pageSize = scrollRange;
-  bool horizontalChange = [topContainer setHorizontalScrollRange: scrollRange page: pageSize];
-  
-  return verticalChange || horizontalChange;
+bool ScintillaCocoa::SetScrollingSize(void) {
+	bool changes = false;
+	InnerView *inner = ContentView();
+	if (!enteredSetScrollingSize) {
+		enteredSetScrollingSize = true;
+		NSScrollView *scrollView = ScrollContainer();
+		NSClipView *clipView = [ScrollContainer() contentView];
+		NSRect clipRect = [clipView bounds];
+		int docHeight = (cs.LinesDisplayed()+1) * vs.lineHeight;
+		if (!endAtLastLine)
+			docHeight += (int([scrollView bounds].size.height / vs.lineHeight)-3) * vs.lineHeight;
+		// Allow extra space so that last scroll position places whole line at top
+		int clipExtra = int(clipRect.size.height) % vs.lineHeight;
+		docHeight += clipExtra;
+		// Ensure all of clipRect covered by Scintilla drawing
+		if (docHeight < clipRect.size.height)
+			docHeight = clipRect.size.height;
+		int docWidth = scrollWidth;
+		bool showHorizontalScroll = horizontalScrollBarVisible &&
+			(wrapState == eWrapNone);
+		if (!showHorizontalScroll)
+			docWidth = clipRect.size.width;
+		NSRect contentRect = {0, 0, docWidth, docHeight};
+		NSRect contentRectNow = [inner frame];
+		changes = (contentRect.size.width != contentRectNow.size.width) ||
+			(contentRect.size.height != contentRectNow.size.height);
+		if (changes) {
+			[inner setFrame: contentRect];
+		}
+		[scrollView setHasVerticalScroller: verticalScrollBarVisible];
+		[scrollView setHasHorizontalScroller: showHorizontalScroll];
+		SetVerticalScrollPos();
+		enteredSetScrollingSize = false;
+	}
+	[inner.owner setMarginWidth: vs.fixedColumnWidth];
+	return changes;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 void ScintillaCocoa::Resize()
 {
+  SetScrollingSize();
   ChangeSize();
 }
 
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Called by the frontend control when the user manipulates one of the scrollers.
- *
- * @param position The relative position of the scroller in the range of [0..1].
- * @param part Specifies which part was clicked on by the user, so we can handle thumb tracking
- *             as well as page and line scrolling.
- * @param horizontal True if the horizontal scroller was hit, otherwise false.
+ * Update fields to match scroll position after receiving a notification that the user has scrolled.
  */
-void ScintillaCocoa::DoScroll(float position, NSScrollerPart part, bool horizontal)
-{
-  // If the given scroller part is not the knob (or knob slot) then the given position is not yet
-  // current and we have to update it.
-  if (horizontal)
-  {
-    // Horizontal offset is given in pixels.
-    PRectangle textRect = GetTextRectangle();
-    int offset = (int) (position * (scrollWidth - textRect.Width()));
-    int smallChange = (int) (textRect.Width() / 30);
-    if (smallChange < 5)
-      smallChange = 5;
-    switch (part)
-    {
-      case NSScrollerDecrementLine:
-        offset -= smallChange;
-        break;
-      case NSScrollerDecrementPage:
-        offset -= textRect.Width();
-        break;
-      case NSScrollerIncrementLine:
-        offset += smallChange;
-        break;
-      case NSScrollerIncrementPage:
-        offset += textRect.Width();
-        break;
-    };
-    HorizontalScrollTo(offset);
-  }
-  else
-  {
-    // VerticalScrolling is by line. If the user is scrolling using the knob we can directly
-    // set the new scroll position. Otherwise we have to compute it first.
-    if (part == NSScrollerKnob)
-      ScrollTo(position * MaxScrollPos(), false);
-    else
-    {
-    switch (part)
-    {
-      case NSScrollerDecrementLine:
-          ScrollTo(topLine - 1, true);
-        break;
-      case NSScrollerDecrementPage:
-          ScrollTo(topLine - LinesOnScreen(), true);
-        break;
-      case NSScrollerIncrementLine:
-          ScrollTo(topLine + 1, true);
-        break;
-      case NSScrollerIncrementPage:
-          ScrollTo(topLine + LinesOnScreen(), true);
-        break;
-    };
-      
-    }
-  }
+void ScintillaCocoa::UpdateForScroll() {
+  Point ptOrigin = GetVisibleOriginInMain();
+  xOffset = ptOrigin.x;
+  int newTop = Platform::Minimum(ptOrigin.y / vs.lineHeight, MaxScrollPos());
+  SetTopLine(newTop);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1562,9 +1795,9 @@ void ScintillaCocoa::IdleTimerFired()
  * @param rect The area to paint, given in the sender's coordinate system.
  * @param gc The context we can use to paint.
  */
-void ScintillaCocoa::Draw(NSRect rect, CGContextRef gc)
+bool ScintillaCocoa::Draw(NSRect rect, CGContextRef gc)
 {
-  SyncPaint(gc, NSRectToPRectangle(rect));
+  return SyncPaint(gc, NSRectToPRectangle(rect));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1666,7 +1899,7 @@ int ScintillaCocoa::InsertText(NSString* input)
 {
   CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
                                                          vs.styles[STYLE_DEFAULT].characterSet);
-  CFRange rangeAll = {0, [input length]};
+  CFRange rangeAll = {0, static_cast<CFIndex>([input length])};
   CFIndex usedLen = 0;
   CFStringGetBytes((CFStringRef)input, rangeAll, encoding, '?',
                    false, NULL, 0, &usedLen);
@@ -1683,6 +1916,21 @@ int ScintillaCocoa::InsertText(NSString* input)
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Used to ensure that only one selection is active for input composition as composition
+ * does not support multi-typing.
+ * Also drop virtual space as that is not supported by composition.
+ */
+void ScintillaCocoa::SelectOnlyMainSelection()
+{
+  SelectionRange mainSel = sel.RangeMain();
+  mainSel.ClearVirtualSpace();
+  sel.SetSelection(mainSel);
+  Redraw();
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Called by the owning view when the mouse pointer enters the control.
  */
@@ -1723,7 +1971,7 @@ void ScintillaCocoa::MouseDown(NSEvent* event)
 void ScintillaCocoa::MouseMove(NSEvent* event)
 {
   lastMouseEvent = event;
-  
+
   ButtonMove(ConvertPoint([event locationInWindow]));
 }
 
@@ -1742,11 +1990,8 @@ void ScintillaCocoa::MouseUp(NSEvent* event)
 void ScintillaCocoa::MouseWheel(NSEvent* event)
 {
   bool command = ([event modifierFlags] & NSCommandKeyMask) != 0;
-  int dX = 0;
   int dY = 0;
 
-  dX = 10 * [event deltaX]; // Arbitrary scale factor.
-
     // In order to make scrolling with larger offset smoother we scroll less lines the larger the 
     // delta value is.
     if ([event deltaY] < 0)
@@ -1765,8 +2010,6 @@ void ScintillaCocoa::MouseWheel(NSEvent* event)
   }
   else
   {
-    HorizontalScrollTo(xOffset - dX);
-    ScrollTo(topLine - dY, true);
   }
 }
 
@@ -1840,3 +2083,81 @@ void ScintillaCocoa::ActiveStateChanged(bool isActive)
 
 //--------------------------------------------------------------------------------------------------
 
+void ScintillaCocoa::ShowFindIndicatorForRange(NSRange charRange, BOOL retaining)
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  NSView *content = ContentView();
+  if (!layerFindIndicator)
+  {
+    layerFindIndicator = [[FindHighlightLayer alloc] init];
+    [content setWantsLayer: YES];
+    layerFindIndicator.geometryFlipped = content.layer.geometryFlipped;
+    [[content layer] addSublayer:layerFindIndicator];
+  }
+  [layerFindIndicator removeAnimationForKey:@"animateFound"];
+  
+  if (charRange.length)
+  {
+    CFStringEncoding encoding = EncodingFromCharacterSet(IsUnicodeMode(),
+							 vs.styles[STYLE_DEFAULT].characterSet);
+    std::vector<char> buffer(charRange.length);
+    pdoc->GetCharRange(&buffer[0], charRange.location, charRange.length);
+    
+    CFStringRef cfsFind = CFStringCreateWithBytes(kCFAllocatorDefault,
+						  reinterpret_cast<const UInt8 *>(&buffer[0]), 
+						  charRange.length, encoding, false);
+    layerFindIndicator.sFind = (NSString *)cfsFind;
+    if (cfsFind)
+        CFRelease(cfsFind);
+    layerFindIndicator.retaining = retaining;
+    layerFindIndicator.positionFind = charRange.location;
+    int style = WndProc(SCI_GETSTYLEAT, charRange.location, 0);
+    std::vector<char> bufferFontName(WndProc(SCI_STYLEGETFONT, style, 0) + 1);
+    WndProc(SCI_STYLEGETFONT, style, (sptr_t)&bufferFontName[0]);
+    layerFindIndicator.sFont = [NSString stringWithUTF8String: &bufferFontName[0]];
+    
+    layerFindIndicator.fontSize = WndProc(SCI_STYLEGETSIZEFRACTIONAL, style, 0) / 
+      (float)SC_FONT_SIZE_MULTIPLIER;
+    layerFindIndicator.widthText = WndProc(SCI_POINTXFROMPOSITION, 0, charRange.location + charRange.length) -
+      WndProc(SCI_POINTXFROMPOSITION, 0, charRange.location);
+    layerFindIndicator.heightLine = WndProc(SCI_TEXTHEIGHT, 0, 0);
+    MoveFindIndicatorWithBounce(YES);
+  }
+  else
+  {
+    [layerFindIndicator hideMatch];
+  }
+#endif
+}
+
+void ScintillaCocoa::MoveFindIndicatorWithBounce(BOOL bounce)
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  if (layerFindIndicator)
+  {
+    CGPoint ptText = CGPointMake(
+      WndProc(SCI_POINTXFROMPOSITION, 0, layerFindIndicator.positionFind),
+      WndProc(SCI_POINTYFROMPOSITION, 0, layerFindIndicator.positionFind));
+    ptText.x = ptText.x - vs.fixedColumnWidth + xOffset;
+    ptText.y += topLine * vs.lineHeight;
+    if (!layerFindIndicator.geometryFlipped)
+    {
+      NSView *content = ContentView();
+      ptText.y = content.bounds.size.height - ptText.y;
+    }
+    [layerFindIndicator animateMatch:ptText bounce:bounce];
+  }
+#endif
+}
+
+void ScintillaCocoa::HideFindIndicator()
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_5
+  if (layerFindIndicator)
+  {
+    [layerFindIndicator hideMatch];
+  }
+#endif
+}
+
+
