@@ -734,6 +734,8 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 	private:
 		FILE *fp;
 		long *offsetList, tableSize;
+		// Private so PDFObjectTracker objects can not be copied
+		PDFObjectTracker(const PDFObjectTracker &) {}
 	public:
 		int index;
 		PDFObjectTracker(FILE *fp_) {
@@ -811,6 +813,8 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 		int styleCurrent, stylePrev;
 		double leading;
 		char *buffer;
+		// Private so PDFRender objects can not be copied
+		PDFRender(const PDFRender &) {}
 	public:
 		PDFObjectTracker *oT;
 		PDFStyle *style;
@@ -821,8 +825,21 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 		//
 		PDFRender() {
 			pageStarted = false;
+			firstLine = false;
 			pageCount = 0;
+			pageContentStart = 0;
+			xPos = 0.0;
+			yPos = 0.0;
+			justWhiteSpace = true;
+			styleCurrent = STYLE_DEFAULT;
+			stylePrev = STYLE_DEFAULT;
+			leading = PDF_FONTSIZE_DEFAULT * PDF_SPACING_DEFAULT;
+			oT = NULL;
 			style = NULL;
+			fontSize = 0;
+			fontSet = PDF_FONT_DEFAULT;
+			pageWidth = 100;
+			pageHeight = 100;
 			buffer = new char[250];
 			segStyle = new char[100];
 		}
@@ -870,7 +887,7 @@ void SciTEBase::SaveToPDF(FilePath saveName) {
 			}
 			// start to write PDF file here (PDF1.4Ref(p63))
 			// ASCII>127 characters to indicate binary-possible stream
-			oT->write("%PDF-1.3\n%쏢\n");
+			oT->write("%PDF-1.3\n%\xc7\xec\x8f\xa2\n");
 			styleCurrent = STYLE_DEFAULT;
 
 			// build objects for font resources; note that font objects are
@@ -1236,7 +1253,6 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 	if (tabSize == 0)
 		tabSize = 4;
 
-	char key[200];
 	int lengthDoc = LengthDocument();
 	TextReader acc(wEditor);
 	bool styleIsUsed[STYLE_MAX + 1];
@@ -1264,6 +1280,7 @@ void SciTEBase::SaveToTEX(FilePath saveName) {
 
 		for (i = 0; i < STYLE_MAX; i++) {      // get keys
 			if (styleIsUsed[i]) {
+				char key[200];
 				sprintf(key, "style.*.%0d", i);
 				char *valdef = StringDup(props.GetExpanded(key).c_str());
 				sprintf(key, "style.%s.%0d", language.c_str(), i);
