@@ -126,6 +126,11 @@ int IFaceTable::GetConstantName(int value, char *nameOut, unsigned nameBufferLen
 	return 0;
 }
 
+#if defined(__GNUC__)
+#if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 7))
+#pragma GCC diagnostic ignored "-Wnarrowing"
+#endif
+#endif
 
 //++Autogenerated -- start of section automatically generated from Scintilla.iface
 
@@ -146,6 +151,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"IDM_ABBREV",242},
 	{"IDM_ABOUT",902},
 	{"IDM_ACTIVATE",320},
+	{"IDM_ALLOWACCESS",119},
 	{"IDM_BLOCK_COMMENT",243},
 	{"IDM_BOOKMARK_CLEARALL",224},
 	{"IDM_BOOKMARK_NEXT",221},
@@ -309,6 +315,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"INDIC_ROUNDBOX",7},
 	{"INDIC_SQUIGGLE",1},
 	{"INDIC_SQUIGGLELOW",11},
+	{"INDIC_SQUIGGLEPIXMAP",13},
 	{"INDIC_STRAIGHTBOX",8},
 	{"INDIC_STRIKE",4},
 	{"INDIC_TT",2},
@@ -477,6 +484,21 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_AVE_WORD4",14},
 	{"SCE_AVE_WORD5",15},
 	{"SCE_AVE_WORD6",16},
+	{"SCE_AVS_CLIPPROP",13},
+	{"SCE_AVS_COMMENTBLOCK",1},
+	{"SCE_AVS_COMMENTBLOCKN",2},
+	{"SCE_AVS_COMMENTLINE",3},
+	{"SCE_AVS_DEFAULT",0},
+	{"SCE_AVS_FILTER",10},
+	{"SCE_AVS_FUNCTION",12},
+	{"SCE_AVS_IDENTIFIER",6},
+	{"SCE_AVS_KEYWORD",9},
+	{"SCE_AVS_NUMBER",4},
+	{"SCE_AVS_OPERATOR",5},
+	{"SCE_AVS_PLUGIN",11},
+	{"SCE_AVS_STRING",7},
+	{"SCE_AVS_TRIPLESTRING",8},
+	{"SCE_AVS_USERDFN",14},
 	{"SCE_BAAN_COMMENT",1},
 	{"SCE_BAAN_COMMENTDOC",2},
 	{"SCE_BAAN_DEFAULT",0},
@@ -638,6 +660,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_CSS_UNKNOWN_IDENTIFIER",7},
 	{"SCE_CSS_UNKNOWN_PSEUDOCLASS",4},
 	{"SCE_CSS_VALUE",8},
+	{"SCE_CSS_VARIABLE",23},
 	{"SCE_C_CHARACTER",7},
 	{"SCE_C_COMMENT",1},
 	{"SCE_C_COMMENTDOC",3},
@@ -652,6 +675,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_C_NUMBER",4},
 	{"SCE_C_OPERATOR",10},
 	{"SCE_C_PREPROCESSOR",9},
+	{"SCE_C_PREPROCESSORCOMMENT",23},
 	{"SCE_C_REGEX",14},
 	{"SCE_C_STRING",6},
 	{"SCE_C_STRINGEOL",12},
@@ -692,6 +716,34 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_D_WORD5",20},
 	{"SCE_D_WORD6",21},
 	{"SCE_D_WORD7",22},
+	{"SCE_ECL_ADDED",24},
+	{"SCE_ECL_CHANGED",26},
+	{"SCE_ECL_CHARACTER",7},
+	{"SCE_ECL_COMMENT",1},
+	{"SCE_ECL_COMMENTDOC",23},
+	{"SCE_ECL_COMMENTDOCKEYWORD",17},
+	{"SCE_ECL_COMMENTDOCKEYWORDERROR",18},
+	{"SCE_ECL_COMMENTLINE",2},
+	{"SCE_ECL_COMMENTLINEDOC",15},
+	{"SCE_ECL_DEFAULT",0},
+	{"SCE_ECL_DELETED",25},
+	{"SCE_ECL_IDENTIFIER",11},
+	{"SCE_ECL_MOVED",27},
+	{"SCE_ECL_NUMBER",3},
+	{"SCE_ECL_OPERATOR",6},
+	{"SCE_ECL_PREPROCESSOR",9},
+	{"SCE_ECL_REGEX",14},
+	{"SCE_ECL_STRING",4},
+	{"SCE_ECL_STRINGEOL",12},
+	{"SCE_ECL_UNKNOWN",10},
+	{"SCE_ECL_UUID",8},
+	{"SCE_ECL_VERBATIM",13},
+	{"SCE_ECL_WORD0",5},
+	{"SCE_ECL_WORD1",16},
+	{"SCE_ECL_WORD2",19},
+	{"SCE_ECL_WORD3",20},
+	{"SCE_ECL_WORD4",21},
+	{"SCE_ECL_WORD5",22},
 	{"SCE_EIFFEL_CHARACTER",5},
 	{"SCE_EIFFEL_COMMENTLINE",1},
 	{"SCE_EIFFEL_DEFAULT",0},
@@ -738,6 +790,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_ERR_DIFF_MESSAGE",13},
 	{"SCE_ERR_ELF",15},
 	{"SCE_ERR_GCC",2},
+	{"SCE_ERR_GCC_INCLUDED_FROM",22},
 	{"SCE_ERR_IFC",16},
 	{"SCE_ERR_IFORT",17},
 	{"SCE_ERR_JAVA_STACK",20},
@@ -1186,6 +1239,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_MYSQL_MAJORKEYWORD",7},
 	{"SCE_MYSQL_NUMBER",6},
 	{"SCE_MYSQL_OPERATOR",14},
+	{"SCE_MYSQL_PLACEHOLDER",22},
 	{"SCE_MYSQL_PROCEDUREKEYWORD",10},
 	{"SCE_MYSQL_QUOTEDIDENTIFIER",17},
 	{"SCE_MYSQL_SQSTRING",12},
@@ -1235,6 +1289,25 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_OPAL_SORT",5},
 	{"SCE_OPAL_SPACE",0},
 	{"SCE_OPAL_STRING",6},
+	{"SCE_OSCRIPT_BLOCK_COMMENT",2},
+	{"SCE_OSCRIPT_CONSTANT",8},
+	{"SCE_OSCRIPT_DEFAULT",0},
+	{"SCE_OSCRIPT_DOC_COMMENT",3},
+	{"SCE_OSCRIPT_DOUBLEQUOTE_STRING",7},
+	{"SCE_OSCRIPT_FUNCTION",15},
+	{"SCE_OSCRIPT_GLOBAL",10},
+	{"SCE_OSCRIPT_IDENTIFIER",9},
+	{"SCE_OSCRIPT_KEYWORD",11},
+	{"SCE_OSCRIPT_LABEL",13},
+	{"SCE_OSCRIPT_LINE_COMMENT",1},
+	{"SCE_OSCRIPT_METHOD",18},
+	{"SCE_OSCRIPT_NUMBER",5},
+	{"SCE_OSCRIPT_OBJECT",16},
+	{"SCE_OSCRIPT_OPERATOR",12},
+	{"SCE_OSCRIPT_PREPROCESSOR",4},
+	{"SCE_OSCRIPT_PROPERTY",17},
+	{"SCE_OSCRIPT_SINGLEQUOTE_STRING",6},
+	{"SCE_OSCRIPT_TYPE",14},
 	{"SCE_PAS_ASM",14},
 	{"SCE_PAS_CHARACTER",12},
 	{"SCE_PAS_COMMENT",2},
@@ -1353,13 +1426,20 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_POWERSHELL_VARIABLE",5},
 	{"SCE_PO_COMMENT",1},
 	{"SCE_PO_DEFAULT",0},
+	{"SCE_PO_ERROR",15},
+	{"SCE_PO_FLAGS",11},
 	{"SCE_PO_FUZZY",8},
 	{"SCE_PO_MSGCTXT",6},
 	{"SCE_PO_MSGCTXT_TEXT",7},
+	{"SCE_PO_MSGCTXT_TEXT_EOL",14},
 	{"SCE_PO_MSGID",2},
 	{"SCE_PO_MSGID_TEXT",3},
+	{"SCE_PO_MSGID_TEXT_EOL",12},
 	{"SCE_PO_MSGSTR",4},
 	{"SCE_PO_MSGSTR_TEXT",5},
+	{"SCE_PO_MSGSTR_TEXT_EOL",13},
+	{"SCE_PO_PROGRAMMER_COMMENT",9},
+	{"SCE_PO_REFERENCE",10},
 	{"SCE_PROPS_ASSIGNMENT",3},
 	{"SCE_PROPS_COMMENT",1},
 	{"SCE_PROPS_DEFAULT",0},
@@ -1635,6 +1715,17 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_TCL_WORD7",18},
 	{"SCE_TCL_WORD8",19},
 	{"SCE_TCL_WORD_IN_QUOTE",4},
+	{"SCE_TCMD_CLABEL",10},
+	{"SCE_TCMD_COMMAND",5},
+	{"SCE_TCMD_COMMENT",1},
+	{"SCE_TCMD_DEFAULT",0},
+	{"SCE_TCMD_ENVIRONMENT",8},
+	{"SCE_TCMD_EXPANSION",9},
+	{"SCE_TCMD_HIDE",4},
+	{"SCE_TCMD_IDENTIFIER",6},
+	{"SCE_TCMD_LABEL",3},
+	{"SCE_TCMD_OPERATOR",7},
+	{"SCE_TCMD_WORD",2},
 	{"SCE_TEX_COMMAND",4},
 	{"SCE_TEX_DEFAULT",0},
 	{"SCE_TEX_GROUP",2},
@@ -1682,6 +1773,29 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCE_VHDL_STRING",4},
 	{"SCE_VHDL_STRINGEOL",7},
 	{"SCE_VHDL_USERWORD",14},
+	{"SCE_VISUALPROLOG_ANONYMOUS",10},
+	{"SCE_VISUALPROLOG_CHARACTER",13},
+	{"SCE_VISUALPROLOG_CHARACTER_ESCAPE_ERROR",15},
+	{"SCE_VISUALPROLOG_CHARACTER_TOO_MANY",14},
+	{"SCE_VISUALPROLOG_COMMENT_BLOCK",4},
+	{"SCE_VISUALPROLOG_COMMENT_KEY",6},
+	{"SCE_VISUALPROLOG_COMMENT_KEY_ERROR",7},
+	{"SCE_VISUALPROLOG_COMMENT_LINE",5},
+	{"SCE_VISUALPROLOG_DEFAULT",0},
+	{"SCE_VISUALPROLOG_IDENTIFIER",8},
+	{"SCE_VISUALPROLOG_KEY_DIRECTIVE",3},
+	{"SCE_VISUALPROLOG_KEY_MAJOR",1},
+	{"SCE_VISUALPROLOG_KEY_MINOR",2},
+	{"SCE_VISUALPROLOG_NUMBER",11},
+	{"SCE_VISUALPROLOG_OPERATOR",12},
+	{"SCE_VISUALPROLOG_STRING",16},
+	{"SCE_VISUALPROLOG_STRING_EOL_OPEN",19},
+	{"SCE_VISUALPROLOG_STRING_ESCAPE",17},
+	{"SCE_VISUALPROLOG_STRING_ESCAPE_ERROR",18},
+	{"SCE_VISUALPROLOG_STRING_VERBATIM",20},
+	{"SCE_VISUALPROLOG_STRING_VERBATIM_EOL",22},
+	{"SCE_VISUALPROLOG_STRING_VERBATIM_SPECIAL",21},
+	{"SCE_VISUALPROLOG_VARIABLE",9},
 	{"SCE_V_COMMENT",1},
 	{"SCE_V_COMMENTLINE",2},
 	{"SCE_V_COMMENTLINEBANG",3},
@@ -1714,13 +1828,20 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_ANNOTATIONGETLINES",2546},
 	{"SCI_ANNOTATIONGETSTYLE",2543},
 	{"SCI_ANNOTATIONGETSTYLEOFFSET",2551},
+	{"SCI_ANNOTATIONGETSTYLES",2545},
+	{"SCI_ANNOTATIONGETTEXT",2541},
 	{"SCI_ANNOTATIONGETVISIBLE",2549},
 	{"SCI_ANNOTATIONSETSTYLE",2542},
 	{"SCI_ANNOTATIONSETSTYLEOFFSET",2550},
+	{"SCI_ANNOTATIONSETSTYLES",2544},
+	{"SCI_ANNOTATIONSETTEXT",2540},
 	{"SCI_ANNOTATIONSETVISIBLE",2548},
 	{"SCI_AUTOCGETAUTOHIDE",2119},
 	{"SCI_AUTOCGETCANCELATSTART",2111},
+	{"SCI_AUTOCGETCASEINSENSITIVEBEHAVIOUR",2635},
 	{"SCI_AUTOCGETCHOOSESINGLE",2114},
+	{"SCI_AUTOCGETCURRENT",2445},
+	{"SCI_AUTOCGETCURRENTTEXT",2610},
 	{"SCI_AUTOCGETDROPRESTOFWORD",2271},
 	{"SCI_AUTOCGETIGNORECASE",2116},
 	{"SCI_AUTOCGETMAXHEIGHT",2211},
@@ -1729,6 +1850,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_AUTOCGETTYPESEPARATOR",2285},
 	{"SCI_AUTOCSETAUTOHIDE",2118},
 	{"SCI_AUTOCSETCANCELATSTART",2110},
+	{"SCI_AUTOCSETCASEINSENSITIVEBEHAVIOUR",2634},
 	{"SCI_AUTOCSETCHOOSESINGLE",2113},
 	{"SCI_AUTOCSETDROPRESTOFWORD",2270},
 	{"SCI_AUTOCSETFILLUPS",2112},
@@ -1742,6 +1864,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_CALLTIPSETFOREHLT",2207},
 	{"SCI_CALLTIPSETPOSITION",2213},
 	{"SCI_CALLTIPUSESTYLE",2212},
+	{"SCI_DISTANCETOSECONDARYSTYLES",4025},
 	{"SCI_GETADDITIONALCARETFORE",2605},
 	{"SCI_GETADDITIONALCARETSBLINK",2568},
 	{"SCI_GETADDITIONALCARETSVISIBLE",2609},
@@ -1755,6 +1878,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETCARETLINEBACK",2097},
 	{"SCI_GETCARETLINEBACKALPHA",2471},
 	{"SCI_GETCARETLINEVISIBLE",2095},
+	{"SCI_GETCARETLINEVISIBLEALWAYS",2654},
 	{"SCI_GETCARETPERIOD",2075},
 	{"SCI_GETCARETSTICKY",2457},
 	{"SCI_GETCARETSTYLE",2513},
@@ -1783,6 +1907,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETFOLDLEVEL",2223},
 	{"SCI_GETFOLDPARENT",2225},
 	{"SCI_GETFONTQUALITY",2612},
+	{"SCI_GETGAPPOSITION",2644},
 	{"SCI_GETHIGHLIGHTGUIDE",2135},
 	{"SCI_GETHOTSPOTACTIVEUNDERLINE",2496},
 	{"SCI_GETHOTSPOTSINGLELINE",2497},
@@ -1796,8 +1921,12 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETLAYOUTCACHE",2273},
 	{"SCI_GETLENGTH",2006},
 	{"SCI_GETLEXER",4002},
+	{"SCI_GETLEXERLANGUAGE",4012},
 	{"SCI_GETLINECOUNT",2154},
 	{"SCI_GETLINEENDPOSITION",2136},
+	{"SCI_GETLINEENDTYPESACTIVE",2658},
+	{"SCI_GETLINEENDTYPESALLOWED",2657},
+	{"SCI_GETLINEENDTYPESSUPPORTED",4018},
 	{"SCI_GETLINEINDENTATION",2127},
 	{"SCI_GETLINEINDENTPOSITION",2128},
 	{"SCI_GETLINESTATE",2093},
@@ -1824,7 +1953,10 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETPRINTCOLOURMODE",2149},
 	{"SCI_GETPRINTMAGNIFICATION",2147},
 	{"SCI_GETPRINTWRAPMODE",2407},
+	{"SCI_GETPROPERTY",4008},
+	{"SCI_GETPROPERTYEXPANDED",4009},
 	{"SCI_GETPROPERTYINT",4010},
+	{"SCI_GETPUNCTUATIONCHARS",2649},
 	{"SCI_GETREADONLY",2140},
 	{"SCI_GETRECTANGULARSELECTIONANCHOR",2591},
 	{"SCI_GETRECTANGULARSELECTIONANCHORVIRTUALSPACE",2595},
@@ -1835,6 +1967,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETSCROLLWIDTHTRACKING",2517},
 	{"SCI_GETSEARCHFLAGS",2199},
 	{"SCI_GETSELALPHA",2477},
+	{"SCI_GETSELECTIONEMPTY",2650},
 	{"SCI_GETSELECTIONEND",2145},
 	{"SCI_GETSELECTIONMODE",2423},
 	{"SCI_GETSELECTIONNANCHOR",2579},
@@ -1850,8 +1983,12 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETSTYLEAT",2010},
 	{"SCI_GETSTYLEBITS",2091},
 	{"SCI_GETSTYLEBITSNEEDED",4011},
+	{"SCI_GETSUBSTYLEBASES",4026},
+	{"SCI_GETSUBSTYLESLENGTH",4022},
+	{"SCI_GETSUBSTYLESSTART",4021},
 	{"SCI_GETTABINDENTS",2261},
 	{"SCI_GETTABWIDTH",2121},
+	{"SCI_GETTAG",2616},
 	{"SCI_GETTARGETEND",2193},
 	{"SCI_GETTARGETSTART",2191},
 	{"SCI_GETTECHNOLOGY",2631},
@@ -1863,7 +2000,9 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_GETVIEWWS",2020},
 	{"SCI_GETVIRTUALSPACEOPTIONS",2597},
 	{"SCI_GETVSCROLLBAR",2281},
+	{"SCI_GETWHITESPACECHARS",2647},
 	{"SCI_GETWHITESPACESIZE",2087},
+	{"SCI_GETWORDCHARS",2646},
 	{"SCI_GETWRAPINDENTMODE",2473},
 	{"SCI_GETWRAPMODE",2269},
 	{"SCI_GETWRAPSTARTINDENT",2465},
@@ -1885,10 +2024,19 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_LINESONSCREEN",2370},
 	{"SCI_MARGINGETSTYLE",2533},
 	{"SCI_MARGINGETSTYLEOFFSET",2538},
+	{"SCI_MARGINGETSTYLES",2535},
+	{"SCI_MARGINGETTEXT",2531},
 	{"SCI_MARGINSETSTYLE",2532},
 	{"SCI_MARGINSETSTYLEOFFSET",2537},
+	{"SCI_MARGINSETSTYLES",2534},
+	{"SCI_MARGINSETTEXT",2530},
+	{"SCI_MARKERSETALPHA",2476},
+	{"SCI_MARKERSETBACK",2042},
+	{"SCI_MARKERSETBACKSELECTED",2292},
+	{"SCI_MARKERSETFORE",2041},
 	{"SCI_OPTIONAL_START",3000},
 	{"SCI_RGBAIMAGESETHEIGHT",2625},
+	{"SCI_RGBAIMAGESETSCALE",2651},
 	{"SCI_RGBAIMAGESETWIDTH",2624},
 	{"SCI_SELECTIONISRECTANGLE",2372},
 	{"SCI_SETADDITIONALCARETFORE",2604},
@@ -1905,6 +2053,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_SETCARETLINEBACK",2098},
 	{"SCI_SETCARETLINEBACKALPHA",2470},
 	{"SCI_SETCARETLINEVISIBLE",2096},
+	{"SCI_SETCARETLINEVISIBLEALWAYS",2655},
 	{"SCI_SETCARETPERIOD",2076},
 	{"SCI_SETCARETSTICKY",2458},
 	{"SCI_SETCARETSTYLE",2512},
@@ -1924,6 +2073,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_SETFIRSTVISIBLELINE",2613},
 	{"SCI_SETFOCUS",2380},
 	{"SCI_SETFOLDEXPANDED",2229},
+	{"SCI_SETFOLDFLAGS",2233},
 	{"SCI_SETFOLDLEVEL",2222},
 	{"SCI_SETFONTQUALITY",2611},
 	{"SCI_SETHIGHLIGHTGUIDE",2134},
@@ -1931,6 +2081,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_SETHOTSPOTSINGLELINE",2421},
 	{"SCI_SETHSCROLLBAR",2130},
 	{"SCI_SETIDENTIFIER",2622},
+	{"SCI_SETIDENTIFIERS",4024},
 	{"SCI_SETINDENT",2122},
 	{"SCI_SETINDENTATIONGUIDES",2132},
 	{"SCI_SETINDICATORCURRENT",2500},
@@ -1939,6 +2090,8 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_SETKEYWORDS",4005},
 	{"SCI_SETLAYOUTCACHE",2272},
 	{"SCI_SETLEXER",4001},
+	{"SCI_SETLEXERLANGUAGE",4006},
+	{"SCI_SETLINEENDTYPESALLOWED",2656},
 	{"SCI_SETLINEINDENTATION",2126},
 	{"SCI_SETLINESTATE",2092},
 	{"SCI_SETMAINSELECTION",2574},
@@ -1962,6 +2115,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_SETPRINTMAGNIFICATION",2146},
 	{"SCI_SETPRINTWRAPMODE",2406},
 	{"SCI_SETPROPERTY",4004},
+	{"SCI_SETPUNCTUATIONCHARS",2648},
 	{"SCI_SETREADONLY",2171},
 	{"SCI_SETRECTANGULARSELECTIONANCHOR",2590},
 	{"SCI_SETRECTANGULARSELECTIONANCHORVIRTUALSPACE",2594},
@@ -2013,6 +2167,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCI_STYLEGETCHANGEABLE",2492},
 	{"SCI_STYLEGETCHARACTERSET",2490},
 	{"SCI_STYLEGETEOLFILLED",2487},
+	{"SCI_STYLEGETFONT",2486},
 	{"SCI_STYLEGETFORE",2481},
 	{"SCI_STYLEGETHOTSPOT",2493},
 	{"SCI_STYLEGETITALIC",2484},
@@ -2066,6 +2221,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCLEX_AU3",60},
 	{"SCLEX_AUTOMATIC",1000},
 	{"SCLEX_AVE",19},
+	{"SCLEX_AVS",104},
 	{"SCLEX_BAAN",31},
 	{"SCLEX_BASH",62},
 	{"SCLEX_BATCH",12},
@@ -2085,6 +2241,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCLEX_CSS",38},
 	{"SCLEX_D",79},
 	{"SCLEX_DIFF",16},
+	{"SCLEX_ECL",105},
 	{"SCLEX_EIFFEL",23},
 	{"SCLEX_EIFFELKW",24},
 	{"SCLEX_ERLANG",53},
@@ -2121,6 +2278,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCLEX_NULL",1},
 	{"SCLEX_OCTAVE",54},
 	{"SCLEX_OPAL",77},
+	{"SCLEX_OSCRIPT",106},
 	{"SCLEX_PASCAL",18},
 	{"SCLEX_PERL",6},
 	{"SCLEX_PHPSCRIPT",69},
@@ -2149,12 +2307,14 @@ static IFaceConstant ifaceConstants[] = {
 	{"SCLEX_TADS3",70},
 	{"SCLEX_TAL",91},
 	{"SCLEX_TCL",25},
+	{"SCLEX_TCMD",103},
 	{"SCLEX_TEX",49},
 	{"SCLEX_TXT2TAGS",99},
 	{"SCLEX_VB",8},
 	{"SCLEX_VBSCRIPT",28},
 	{"SCLEX_VERILOG",56},
 	{"SCLEX_VHDL",64},
+	{"SCLEX_VISUALPROLOG",107},
 	{"SCLEX_XCODE",13},
 	{"SCLEX_XML",5},
 	{"SCLEX_YAML",48},
@@ -2180,6 +2340,8 @@ static IFaceConstant ifaceConstants[] = {
 	{"SC_CARETSTICKY_OFF",0},
 	{"SC_CARETSTICKY_ON",1},
 	{"SC_CARETSTICKY_WHITESPACE",2},
+	{"SC_CASEINSENSITIVEBEHAVIOUR_IGNORECASE",1},
+	{"SC_CASEINSENSITIVEBEHAVIOUR_RESPECTCASE",0},
 	{"SC_CASE_LOWER",2},
 	{"SC_CASE_MIXED",0},
 	{"SC_CASE_UPPER",1},
@@ -2232,6 +2394,8 @@ static IFaceConstant ifaceConstants[] = {
 	{"SC_IV_NONE",0},
 	{"SC_IV_REAL",1},
 	{"SC_LASTSTEPINUNDOREDO",0x100},
+	{"SC_LINE_END_TYPE_DEFAULT",0},
+	{"SC_LINE_END_TYPE_UNICODE",1},
 	{"SC_MARGINOPTION_NONE",0},
 	{"SC_MARGINOPTION_SUBLINESELECT",1},
 	{"SC_MARGIN_BACK",2},
@@ -2279,7 +2443,8 @@ static IFaceConstant ifaceConstants[] = {
 	{"SC_MARK_TCORNERCURVE",17},
 	{"SC_MARK_UNDERLINE",29},
 	{"SC_MARK_VLINE",9},
-	{"SC_MASK_FOLDERS",0xFE000000},
+	{"SC_MASK_FOLDERS",static_cast<int>(0xFE000000)},
+	{"SC_MAX_MARGIN",4},
 	{"SC_MODEVENTMASKALL",0xFFFFF},
 	{"SC_MOD_BEFOREDELETE",0x800},
 	{"SC_MOD_BEFOREINSERT",0x400},
@@ -2334,6 +2499,7 @@ static IFaceConstant ifaceConstants[] = {
 	{"SC_WRAPVISUALFLAGLOC_END_BY_TEXT",0x0001},
 	{"SC_WRAPVISUALFLAGLOC_START_BY_TEXT",0x0002},
 	{"SC_WRAPVISUALFLAG_END",0x0001},
+	{"SC_WRAPVISUALFLAG_MARGIN",0x0004},
 	{"SC_WRAPVISUALFLAG_NONE",0x0000},
 	{"SC_WRAPVISUALFLAG_START",0x0002},
 	{"SC_WRAP_CHAR",2},
@@ -2360,18 +2526,14 @@ static IFaceFunction ifaceFunctions[] = {
 	{"AddText", 2001, iface_void, {iface_length, iface_string}},
 	{"AddUndoAction", 2560, iface_void, {iface_int, iface_int}},
 	{"Allocate", 2446, iface_void, {iface_int, iface_void}},
+	{"AllocateExtendedStyles", 2553, iface_int, {iface_int, iface_void}},
+	{"AllocateSubStyles", 4020, iface_int, {iface_int, iface_int}},
 	{"AnnotationClearAll", 2547, iface_void, {iface_void, iface_void}},
-	{"AnnotationGetStyles", 2545, iface_int, {iface_int, iface_stringresult}},
-	{"AnnotationGetText", 2541, iface_int, {iface_int, iface_stringresult}},
-	{"AnnotationSetStyles", 2544, iface_void, {iface_int, iface_string}},
-	{"AnnotationSetText", 2540, iface_void, {iface_int, iface_string}},
 	{"AppendText", 2282, iface_void, {iface_length, iface_string}},
 	{"AssignCmdKey", 2070, iface_void, {iface_keymod, iface_int}},
 	{"AutoCActive", 2102, iface_bool, {iface_void, iface_void}},
 	{"AutoCCancel", 2101, iface_void, {iface_void, iface_void}},
 	{"AutoCComplete", 2104, iface_void, {iface_void, iface_void}},
-	{"AutoCGetCurrent", 2445, iface_int, {iface_void, iface_void}},
-	{"AutoCGetCurrentText", 2610, iface_int, {iface_void, iface_stringresult}},
 	{"AutoCPosStart", 2103, iface_position, {iface_void, iface_void}},
 	{"AutoCSelect", 2108, iface_void, {iface_void, iface_string}},
 	{"AutoCShow", 2100, iface_void, {iface_int, iface_string}},
@@ -2427,6 +2589,7 @@ static IFaceFunction ifaceFunctions[] = {
 	{"DelWordRightEnd", 2518, iface_void, {iface_void, iface_void}},
 	{"DeleteBack", 2326, iface_void, {iface_void, iface_void}},
 	{"DeleteBackNotLine", 2344, iface_void, {iface_void, iface_void}},
+	{"DeleteRange", 2645, iface_void, {iface_position, iface_int}},
 	{"DescribeKeyWordSets", 4017, iface_int, {iface_void, iface_stringresult}},
 	{"DescribeProperty", 4016, iface_int, {iface_string, iface_stringresult}},
 	{"DocLineFromVisible", 2221, iface_int, {iface_int, iface_void}},
@@ -2441,22 +2604,23 @@ static IFaceFunction ifaceFunctions[] = {
 	{"EnsureVisible", 2232, iface_void, {iface_int, iface_void}},
 	{"EnsureVisibleEnforcePolicy", 2234, iface_void, {iface_int, iface_void}},
 	{"FindColumn", 2456, iface_int, {iface_int, iface_int}},
+	{"FindIndicatorFlash", 2641, iface_void, {iface_position, iface_position}},
+	{"FindIndicatorHide", 2642, iface_void, {iface_void, iface_void}},
+	{"FindIndicatorShow", 2640, iface_void, {iface_position, iface_position}},
 	{"FindText", 2150, iface_position, {iface_int, iface_findtext}},
 	{"FormFeed", 2330, iface_void, {iface_void, iface_void}},
 	{"FormatRange", 2151, iface_position, {iface_bool, iface_formatrange}},
+	{"FreeSubStyles", 4023, iface_void, {iface_void, iface_void}},
 	{"GetCurLine", 2027, iface_int, {iface_length, iface_stringresult}},
 	{"GetHotspotActiveBack", 2495, iface_colour, {iface_void, iface_void}},
 	{"GetHotspotActiveFore", 2494, iface_colour, {iface_void, iface_void}},
 	{"GetLastChild", 2224, iface_int, {iface_int, iface_int}},
-	{"GetLexerLanguage", 4012, iface_int, {iface_void, iface_stringresult}},
 	{"GetLine", 2153, iface_int, {iface_int, iface_stringresult}},
 	{"GetLineSelEndPosition", 2425, iface_position, {iface_int, iface_void}},
 	{"GetLineSelStartPosition", 2424, iface_position, {iface_int, iface_void}},
-	{"GetProperty", 4008, iface_int, {iface_string, iface_stringresult}},
-	{"GetPropertyExpanded", 4009, iface_int, {iface_string, iface_stringresult}},
+	{"GetRangePointer", 2643, iface_int, {iface_int, iface_int}},
 	{"GetSelText", 2161, iface_int, {iface_void, iface_stringresult}},
 	{"GetStyledText", 2015, iface_int, {iface_void, iface_textrange}},
-	{"GetTag", 2616, iface_int, {iface_int, iface_stringresult}},
 	{"GetText", 2182, iface_int, {iface_length, iface_stringresult}},
 	{"GetTextRange", 2162, iface_int, {iface_void, iface_textrange}},
 	{"GotoLine", 2024, iface_void, {iface_int, iface_void}},
@@ -2505,10 +2669,6 @@ static IFaceFunction ifaceFunctions[] = {
 	{"LinesSplit", 2289, iface_void, {iface_int, iface_void}},
 	{"LoadLexerLibrary", 4007, iface_void, {iface_void, iface_string}},
 	{"LowerCase", 2340, iface_void, {iface_void, iface_void}},
-	{"MarginGetStyles", 2535, iface_int, {iface_int, iface_stringresult}},
-	{"MarginGetText", 2531, iface_int, {iface_int, iface_stringresult}},
-	{"MarginSetStyles", 2534, iface_void, {iface_int, iface_string}},
-	{"MarginSetText", 2530, iface_void, {iface_int, iface_string}},
 	{"MarginTextClearAll", 2536, iface_void, {iface_void, iface_void}},
 	{"MarkerAdd", 2043, iface_int, {iface_int, iface_int}},
 	{"MarkerAddSet", 2466, iface_void, {iface_int, iface_int}},
@@ -2523,10 +2683,6 @@ static IFaceFunction ifaceFunctions[] = {
 	{"MarkerLineFromHandle", 2017, iface_int, {iface_int, iface_void}},
 	{"MarkerNext", 2047, iface_int, {iface_int, iface_int}},
 	{"MarkerPrevious", 2048, iface_int, {iface_int, iface_int}},
-	{"MarkerSetAlpha", 2476, iface_void, {iface_int, iface_int}},
-	{"MarkerSetBack", 2042, iface_void, {iface_int, iface_colour}},
-	{"MarkerSetBackSelected", 2292, iface_void, {iface_int, iface_colour}},
-	{"MarkerSetFore", 2041, iface_void, {iface_int, iface_colour}},
 	{"MarkerSymbolDefined", 2529, iface_int, {iface_int, iface_void}},
 	{"MoveCaretInsideView", 2401, iface_void, {iface_void, iface_void}},
 	{"MoveSelectedLinesDown", 2621, iface_void, {iface_void, iface_void}},
@@ -2557,12 +2713,14 @@ static IFaceFunction ifaceFunctions[] = {
 	{"Redo", 2011, iface_void, {iface_void, iface_void}},
 	{"RegisterImage", 2405, iface_void, {iface_int, iface_string}},
 	{"RegisterRGBAImage", 2627, iface_void, {iface_int, iface_string}},
+	{"ReleaseAllExtendedStyles", 2552, iface_void, {iface_void, iface_void}},
 	{"ReleaseDocument", 2377, iface_void, {iface_void, iface_int}},
 	{"ReplaceSel", 2170, iface_void, {iface_void, iface_string}},
 	{"ReplaceTarget", 2194, iface_int, {iface_length, iface_string}},
 	{"ReplaceTargetRE", 2195, iface_int, {iface_length, iface_string}},
 	{"RotateSelection", 2606, iface_void, {iface_void, iface_void}},
 	{"ScrollCaret", 2169, iface_void, {iface_void, iface_void}},
+	{"ScrollRange", 2569, iface_void, {iface_position, iface_position}},
 	{"ScrollToEnd", 2629, iface_void, {iface_void, iface_void}},
 	{"ScrollToStart", 2628, iface_void, {iface_void, iface_void}},
 	{"SearchAnchor", 2366, iface_void, {iface_void, iface_void}},
@@ -2573,13 +2731,11 @@ static IFaceFunction ifaceFunctions[] = {
 	{"SelectionDuplicate", 2469, iface_void, {iface_void, iface_void}},
 	{"SetCharsDefault", 2444, iface_void, {iface_void, iface_void}},
 	{"SetEmptySelection", 2556, iface_void, {iface_position, iface_void}},
-	{"SetFoldFlags", 2233, iface_void, {iface_int, iface_void}},
 	{"SetFoldMarginColour", 2290, iface_void, {iface_bool, iface_colour}},
 	{"SetFoldMarginHiColour", 2291, iface_void, {iface_bool, iface_colour}},
 	{"SetHotspotActiveBack", 2411, iface_void, {iface_bool, iface_colour}},
 	{"SetHotspotActiveFore", 2410, iface_void, {iface_bool, iface_colour}},
 	{"SetLengthForEncode", 2448, iface_void, {iface_int, iface_void}},
-	{"SetLexerLanguage", 4006, iface_void, {iface_void, iface_string}},
 	{"SetSavePoint", 2014, iface_void, {iface_void, iface_void}},
 	{"SetSel", 2160, iface_void, {iface_position, iface_position}},
 	{"SetSelBack", 2068, iface_void, {iface_bool, iface_colour}},
@@ -2602,7 +2758,6 @@ static IFaceFunction ifaceFunctions[] = {
 	{"StutteredPageUp", 2435, iface_void, {iface_void, iface_void}},
 	{"StutteredPageUpExtend", 2436, iface_void, {iface_void, iface_void}},
 	{"StyleClearAll", 2050, iface_void, {iface_void, iface_void}},
-	{"StyleGetFont", 2486, iface_int, {iface_int, iface_stringresult}},
 	{"StyleResetDefault", 2058, iface_void, {iface_void, iface_void}},
 	{"SwapMainAnchorCaret", 2607, iface_void, {iface_void, iface_void}},
 	{"Tab", 2327, iface_void, {iface_void, iface_void}},
@@ -2617,6 +2772,8 @@ static IFaceFunction ifaceFunctions[] = {
 	{"UsePopUp", 2371, iface_void, {iface_bool, iface_void}},
 	{"UserListShow", 2117, iface_void, {iface_int, iface_string}},
 	{"VCHome", 2331, iface_void, {iface_void, iface_void}},
+	{"VCHomeDisplay", 2652, iface_void, {iface_void, iface_void}},
+	{"VCHomeDisplayExtend", 2653, iface_void, {iface_void, iface_void}},
 	{"VCHomeExtend", 2332, iface_void, {iface_void, iface_void}},
 	{"VCHomeRectExtend", 2431, iface_void, {iface_void, iface_void}},
 	{"VCHomeWrap", 2453, iface_void, {iface_void, iface_void}},
@@ -2655,10 +2812,15 @@ static IFaceProperty ifaceProperties[] = {
 	{"AnnotationLines", 2546, 0, iface_int, iface_int},
 	{"AnnotationStyle", 2543, 2542, iface_int, iface_int},
 	{"AnnotationStyleOffset", 2551, 2550, iface_int, iface_void},
+	{"AnnotationStyles", 2545, 2544, iface_stringresult, iface_int},
+	{"AnnotationText", 2541, 2540, iface_stringresult, iface_int},
 	{"AnnotationVisible", 2549, 2548, iface_int, iface_void},
 	{"AutoCAutoHide", 2119, 2118, iface_bool, iface_void},
 	{"AutoCCancelAtStart", 2111, 2110, iface_bool, iface_void},
+	{"AutoCCaseInsensitiveBehaviour", 2635, 2634, iface_int, iface_void},
 	{"AutoCChooseSingle", 2114, 2113, iface_bool, iface_void},
+	{"AutoCCurrent", 2445, 0, iface_int, iface_void},
+	{"AutoCCurrentText", 2610, 0, iface_stringresult, iface_void},
 	{"AutoCDropRestOfWord", 2271, 2270, iface_bool, iface_void},
 	{"AutoCFillUps", 0, 2112, iface_string, iface_void},
 	{"AutoCIgnoreCase", 2116, 2115, iface_bool, iface_void},
@@ -2677,6 +2839,7 @@ static IFaceProperty ifaceProperties[] = {
 	{"CaretLineBack", 2097, 2098, iface_colour, iface_void},
 	{"CaretLineBackAlpha", 2471, 2470, iface_int, iface_void},
 	{"CaretLineVisible", 2095, 2096, iface_bool, iface_void},
+	{"CaretLineVisibleAlways", 2654, 2655, iface_bool, iface_void},
 	{"CaretPeriod", 2075, 2076, iface_int, iface_void},
 	{"CaretSticky", 2457, 2458, iface_int, iface_void},
 	{"CaretStyle", 2513, 2512, iface_int, iface_void},
@@ -2690,6 +2853,7 @@ static IFaceProperty ifaceProperties[] = {
 	{"Cursor", 2387, 2386, iface_int, iface_void},
 	{"DirectFunction", 2184, 0, iface_int, iface_void},
 	{"DirectPointer", 2185, 0, iface_int, iface_void},
+	{"DistanceToSecondaryStyles", 4025, 0, iface_int, iface_void},
 	{"DocPointer", 2357, 2358, iface_int, iface_void},
 	{"EOLMode", 2030, 2031, iface_int, iface_void},
 	{"EdgeColour", 2364, 2365, iface_colour, iface_void},
@@ -2702,14 +2866,17 @@ static IFaceProperty ifaceProperties[] = {
 	{"FirstVisibleLine", 2152, 2613, iface_int, iface_void},
 	{"Focus", 2381, 2380, iface_bool, iface_void},
 	{"FoldExpanded", 2230, 2229, iface_bool, iface_int},
+	{"FoldFlags", 0, 2233, iface_int, iface_void},
 	{"FoldLevel", 2223, 2222, iface_int, iface_int},
 	{"FoldParent", 2225, 0, iface_int, iface_int},
 	{"FontQuality", 2612, 2611, iface_int, iface_void},
+	{"GapPosition", 2644, 0, iface_position, iface_void},
 	{"HScrollBar", 2131, 2130, iface_bool, iface_void},
 	{"HighlightGuide", 2135, 2134, iface_int, iface_void},
 	{"HotspotActiveUnderline", 2496, 2412, iface_bool, iface_void},
 	{"HotspotSingleLine", 2497, 2421, iface_bool, iface_void},
 	{"Identifier", 2623, 2622, iface_int, iface_void},
+	{"Identifiers", 0, 4024, iface_string, iface_int},
 	{"Indent", 2123, 2122, iface_int, iface_void},
 	{"IndentationGuides", 2133, 2132, iface_int, iface_void},
 	{"IndicAlpha", 2524, 2523, iface_int, iface_int},
@@ -2724,8 +2891,12 @@ static IFaceProperty ifaceProperties[] = {
 	{"LayoutCache", 2273, 2272, iface_int, iface_void},
 	{"Length", 2006, 0, iface_int, iface_void},
 	{"Lexer", 4002, 4001, iface_int, iface_void},
+	{"LexerLanguage", 4012, 4006, iface_stringresult, iface_void},
 	{"LineCount", 2154, 0, iface_int, iface_void},
-	{"LineEndPosition", 2136, 0, iface_int, iface_int},
+	{"LineEndPosition", 2136, 0, iface_position, iface_int},
+	{"LineEndTypesActive", 2658, 0, iface_int, iface_void},
+	{"LineEndTypesAllowed", 2657, 2656, iface_int, iface_void},
+	{"LineEndTypesSupported", 4018, 0, iface_int, iface_void},
 	{"LineIndentPosition", 2128, 0, iface_position, iface_int},
 	{"LineIndentation", 2127, 2126, iface_int, iface_int},
 	{"LineState", 2093, 2092, iface_int, iface_int},
@@ -2740,8 +2911,14 @@ static IFaceProperty ifaceProperties[] = {
 	{"MarginSensitiveN", 2247, 2246, iface_bool, iface_int},
 	{"MarginStyle", 2533, 2532, iface_int, iface_int},
 	{"MarginStyleOffset", 2538, 2537, iface_int, iface_void},
+	{"MarginStyles", 2535, 2534, iface_stringresult, iface_int},
+	{"MarginText", 2531, 2530, iface_stringresult, iface_int},
 	{"MarginTypeN", 2241, 2240, iface_int, iface_int},
 	{"MarginWidthN", 2243, 2242, iface_int, iface_int},
+	{"MarkerAlpha", 0, 2476, iface_int, iface_int},
+	{"MarkerBack", 0, 2042, iface_colour, iface_int},
+	{"MarkerBackSelected", 0, 2292, iface_colour, iface_int},
+	{"MarkerFore", 0, 2041, iface_colour, iface_int},
 	{"MaxLineState", 2094, 0, iface_int, iface_void},
 	{"ModEventMask", 2378, 2359, iface_int, iface_void},
 	{"Modify", 2159, 0, iface_bool, iface_void},
@@ -2755,9 +2932,12 @@ static IFaceProperty ifaceProperties[] = {
 	{"PrintColourMode", 2149, 2148, iface_int, iface_void},
 	{"PrintMagnification", 2147, 2146, iface_int, iface_void},
 	{"PrintWrapMode", 2407, 2406, iface_int, iface_void},
-	{"Property", 0, 4004, iface_string, iface_string},
+	{"Property", 4008, 4004, iface_stringresult, iface_string},
+	{"PropertyExpanded", 4009, 0, iface_stringresult, iface_string},
 	{"PropertyInt", 4010, 0, iface_int, iface_string},
+	{"PunctuationChars", 2649, 2648, iface_stringresult, iface_void},
 	{"RGBAImageHeight", 0, 2625, iface_int, iface_void},
+	{"RGBAImageScale", 0, 2651, iface_int, iface_void},
 	{"RGBAImageWidth", 0, 2624, iface_int, iface_void},
 	{"ReadOnly", 2140, 2171, iface_bool, iface_void},
 	{"RectangularSelectionAnchor", 2591, 2590, iface_position, iface_void},
@@ -2770,6 +2950,7 @@ static IFaceProperty ifaceProperties[] = {
 	{"SearchFlags", 2199, 2198, iface_int, iface_void},
 	{"SelAlpha", 2477, 2478, iface_int, iface_void},
 	{"SelEOLFilled", 2479, 2480, iface_bool, iface_void},
+	{"SelectionEmpty", 2650, 0, iface_bool, iface_void},
 	{"SelectionEnd", 2145, 2144, iface_position, iface_void},
 	{"SelectionIsRectangle", 2372, 0, iface_bool, iface_void},
 	{"SelectionMode", 2423, 2422, iface_int, iface_void},
@@ -2791,7 +2972,7 @@ static IFaceProperty ifaceProperties[] = {
 	{"StyleChangeable", 2492, 2099, iface_bool, iface_int},
 	{"StyleCharacterSet", 2490, 2066, iface_int, iface_int},
 	{"StyleEOLFilled", 2487, 2057, iface_bool, iface_int},
-	{"StyleFont", 0, 2056, iface_string, iface_int},
+	{"StyleFont", 2486, 2056, iface_stringresult, iface_int},
 	{"StyleFore", 2481, 2051, iface_colour, iface_int},
 	{"StyleHotSpot", 2493, 2409, iface_bool, iface_int},
 	{"StyleItalic", 2484, 2054, iface_bool, iface_int},
@@ -2800,8 +2981,12 @@ static IFaceProperty ifaceProperties[] = {
 	{"StyleUnderline", 2488, 2059, iface_bool, iface_int},
 	{"StyleVisible", 2491, 2074, iface_bool, iface_int},
 	{"StyleWeight", 2064, 2063, iface_int, iface_int},
+	{"SubStyleBases", 4026, 0, iface_stringresult, iface_void},
+	{"SubStylesLength", 4022, 0, iface_int, iface_int},
+	{"SubStylesStart", 4021, 0, iface_int, iface_int},
 	{"TabIndents", 2261, 2260, iface_bool, iface_void},
 	{"TabWidth", 2121, 2036, iface_int, iface_void},
+	{"Tag", 2616, 0, iface_stringresult, iface_int},
 	{"TargetEnd", 2193, 2192, iface_position, iface_void},
 	{"TargetStart", 2191, 2190, iface_position, iface_void},
 	{"Technology", 2631, 2630, iface_int, iface_void},
@@ -2813,9 +2998,9 @@ static IFaceProperty ifaceProperties[] = {
 	{"ViewEOL", 2355, 2356, iface_bool, iface_void},
 	{"ViewWS", 2020, 2021, iface_int, iface_void},
 	{"VirtualSpaceOptions", 2597, 2596, iface_int, iface_void},
-	{"WhitespaceChars", 0, 2443, iface_string, iface_void},
+	{"WhitespaceChars", 2647, 2443, iface_stringresult, iface_void},
 	{"WhitespaceSize", 2087, 2086, iface_int, iface_void},
-	{"WordChars", 0, 2077, iface_string, iface_void},
+	{"WordChars", 2646, 2077, iface_stringresult, iface_void},
 	{"WrapIndentMode", 2473, 2472, iface_int, iface_void},
 	{"WrapMode", 2269, 2268, iface_int, iface_void},
 	{"WrapStartIndent", 2465, 2464, iface_int, iface_void},
@@ -2826,9 +3011,9 @@ static IFaceProperty ifaceProperties[] = {
 };
 
 enum {
-	ifaceFunctionCount = 286,
-	ifaceConstantCount = 2221,
-	ifacePropertyCount = 180
+	ifaceFunctionCount = 277,
+	ifaceConstantCount = 2382,
+	ifacePropertyCount = 208
 };
 
 //--Autogenerated
