@@ -29,19 +29,23 @@ class Job {
 public:
 	SString command;
 	FilePath directory;
-	SString input;
 	JobSubsystem jobType;
+	SString input;
 	int flags;
 
 	Job() {
 		Clear();
 	}
 
+	Job(const SString &command_, const FilePath &directory_, JobSubsystem jobType_, const SString &input_, int flags_)
+		: command(command_), directory(directory_), jobType(jobType_), input(input_), flags(flags_) {
+	}
+
 	void Clear() {
 		command = "";
 		directory.Init();
-		input = "";
 		jobType = jobCLI;
+		input = "";
 		flags = 0;
 	}
 };
@@ -102,6 +106,10 @@ public:
 		executing = state;
 	}
 
+	bool HasCommandToRun() const {
+		return commandCurrent > 0;
+	}
+
 	long SetCancelFlag(long value) {
 		Lock lock(mutex);
 		long cancelFlagPrevious = cancelFlag;
@@ -113,6 +121,9 @@ public:
 		Lock lock(mutex);
 		return cancelFlag;
 	}
+
+	void ClearJobs();
+	void AddCommand(const SString &command, const FilePath &directory, JobSubsystem jobType, const SString &input, int flags);
 };
 
 #endif
