@@ -118,7 +118,7 @@ inline bool IFaceFunctionIsScriptable(const IFaceFunction &f) {
 }
 
 inline bool IFacePropertyIsScriptable(const IFaceProperty &p) {
-	return (((p.valueType > iface_void) && (p.valueType <= iface_string) && (p.valueType != iface_keymod)) &&
+	return (((p.valueType > iface_void) && (p.valueType <= iface_stringresult) && (p.valueType != iface_keymod)) &&
 	        ((p.paramType < iface_colour) || (p.paramType == iface_string) ||
 	                (p.paramType == iface_bool)) && (p.getter || p.setter));
 }
@@ -291,6 +291,45 @@ static int cf_scite_update_status_bar(lua_State *L) {
 	return 0;
 }
 
+static int cf_scite_strip_show(lua_State *L) {
+	const char *s = luaL_checkstring(L, 1);
+	if (s) {
+		host->UserStripShow(s);
+	}
+	return 0;
+}
+
+static int cf_scite_strip_set(lua_State *L) {
+	int control = luaL_checkint(L, 1);
+	const char *value = luaL_checkstring(L, 2);
+	if (value) {
+		host->UserStripSet(control, value);
+	}
+	return 0;
+}
+
+static int cf_scite_strip_set_list(lua_State *L) {
+	int control = luaL_checkint(L, 1);
+	const char *value = luaL_checkstring(L, 2);
+	if (value) {
+		host->UserStripSetList(control, value);
+	}
+	return 0;
+}
+
+static int cf_scite_strip_value(lua_State *L) {
+	int control = luaL_checkint(L, 1);
+	const char *value = host->UserStripValue(control);
+	if (value) {
+		lua_pushstring(L, value);
+		delete []value;
+		return 1;
+	} else {
+		lua_pushstring(L, "");
+	}
+	return 0;
+}
+
 static ExtensionAPI::Pane check_pane_object(lua_State *L, int index) {
 	ExtensionAPI::Pane *pPane = reinterpret_cast<ExtensionAPI::Pane *>(checkudata(L, index, "SciTE_MT_Pane"));
 
@@ -810,6 +849,21 @@ static bool CallNamedFunction(const char *name, int numberArg, const char *strin
 	return handled;
 }
 
+static bool CallNamedFunction(const char *name, int numberArg, int numberArg2) {
+	bool handled = false;
+	if (luaState) {
+		lua_getglobal(luaState, name);
+		if (lua_isfunction(luaState, -1)) {
+			lua_pushnumber(luaState, numberArg);
+			lua_pushnumber(luaState, numberArg2);
+			handled = call_function(luaState, 2);
+		} else {
+			lua_pop(luaState, 1);
+		}
+	}
+	return handled;
+}
+
 static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 	ExtensionAPI::Pane p = check_pane_object(L, 1);
 
@@ -826,7 +880,7 @@ static int iface_function_helper(lua_State *L, const IFaceFunction &func) {
 		params[0] = lua_strlen(L, arg);
 		params[1] = reinterpret_cast<sptr_t>(params[0] ? lua_tostring(L, arg) : "");
 		loopParamCount = 0;
-	} else if (func.paramType[1] == iface_stringresult) {
+	} else if ((func.paramType[1] == iface_stringresult) || (func.returnType == iface_stringresult)) {
 		needStringResult = true;
 		// The buffer will be allocated later, so it won't leak if Lua does
 		// a longjmp in response to a bad arg.
@@ -1352,6 +1406,18 @@ static bool InitGlobalScope(bool checkProperties, bool forceReload = false) {
 	lua_pushcfunction(luaState, cf_scite_update_status_bar);
 	lua_setfield(luaState, -2, "UpdateStatusBar");
 
+	lua_pushcfunction(luaState, cf_scite_strip_show);
+	lua_setfield(luaState, -2, "StripShow");
+
+	lua_pushcfunction(luaState, cf_scite_strip_set);
+	lua_setfield(luaState, -2, "StripSet");
+
+	lua_pushcfunction(luaState, cf_scite_strip_set_list);
+	lua_setfield(luaState, -2, "StripSetList");
+
+	lua_pushcfunction(luaState, cf_scite_strip_value);
+	lua_setfield(luaState, -2, "StripValue");
+
 	lua_setglobal(luaState, "scite");
 
 	// Metatable for global namespace, to publish iface constants
@@ -1597,9 +1663,8 @@ bool LuaExtension::OnBeforeSave(const char *filename) {
 bool LuaExtension::OnSave(const char *filename) {
 	bool result = CallNamedFunction("OnSave", filename);
 
-	// should this be case insensitive on windows?  check for different
-	// ways of spelling a filename, e.g. short vs. long?
-	if (startupScript && 0 == strcmp(filename, startupScript)) {
+	FilePath fpSaving = FilePath(GUI::StringFromUTF8(filename)).NormalizePath();
+	if (startupScript && fpSaving == FilePath(GUI::StringFromUTF8(startupScript)).NormalizePath()) {
 		if (GetPropertyInt("ext.lua.auto.reload") > 0) {
 			InitGlobalScope(false, true);
 			if (extensionScript.length()) {
@@ -2028,6 +2093,10 @@ bool LuaExtension::OnClose(const char *filename) {
 	return CallNamedFunction("OnClose", filename);
 }
 
+bool LuaExtension::OnUserStrip(int control, int change) {
+	return CallNamedFunction("OnStrip", control, change);
+}
+
 #ifdef _MSC_VER
 // Unreferenced inline functions are OK
 #pragma warning(disable: 4514)
