@@ -13,6 +13,7 @@
 #include <fcntl.h>
 #include <time.h>
 #include <locale.h>
+#include <assert.h>
 
 #include <string>
 #include <vector>
@@ -92,7 +93,6 @@ PropSetFile &PropSetFile::operator=(const PropSetFile &assign) {
 		lowerKeys = assign.lowerKeys;
 		superPS = assign.superPS;
 		props = assign.props;
-		enumnext = "";
 	}
 	return *this;
 }
@@ -327,19 +327,6 @@ void PropSetFile::Clear() {
 	props.clear();
 }
 
-char *PropSetFile::ToString() const {
-	std::string sval;
-	for (mapss::const_iterator it=props.begin(); it != props.end(); ++it) {
-		sval += it->first;
-		sval += "=";
-		sval += it->second;
-		sval += "\n";
-	}
-	char *ret = new char [sval.size() + 1];
-	strcpy(ret, sval.c_str());
-	return ret;
-}
-
 /**
  * Get a line of input. If end of line escaped with '\\' then continue reading.
  */
@@ -445,10 +432,10 @@ bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath direc
 void PropSetFile::ReadFromMemory(const char *data, size_t len, FilePath directoryForImports,
                                  const ImportFilter &filter, std::vector<FilePath> *imports) {
 	const char *pd = data;
-	char lineBuffer[60000];
+	std::vector<char> lineBuffer(len+1);	// +1 for NUL
 	bool ifIsTrue = true;
 	while (len > 0) {
-		GetFullLine(pd, len, lineBuffer, sizeof(lineBuffer));
+		GetFullLine(pd, len, &lineBuffer[0], lineBuffer.size());
 		if (lowerKeys) {
 			for (int i=0; lineBuffer[i] && (lineBuffer[i] != '='); i++) {
 				if ((lineBuffer[i] >= 'A') && (lineBuffer[i] <= 'Z')) {
@@ -456,7 +443,7 @@ void PropSetFile::ReadFromMemory(const char *data, size_t len, FilePath director
 				}
 			}
 		}
-		ifIsTrue = ReadLine(lineBuffer, ifIsTrue, directoryForImports, filter, imports);
+		ifIsTrue = ReadLine(&lineBuffer[0], ifIsTrue, directoryForImports, filter, imports);
 	}
 }
 
@@ -464,10 +451,17 @@ bool PropSetFile::Read(FilePath filename, FilePath directoryForImports,
                        const ImportFilter &filter, std::vector<FilePath> *imports) {
 	FILE *rcfile = filename.Open(fileRead);
 	if (rcfile) {
-		char propsData[60000];
-		int lenFile = static_cast<int>(fread(propsData, 1, sizeof(propsData), rcfile));
+		fseek(rcfile, 0, SEEK_END);
+		long sizeFile = ftell(rcfile);
+		if (sizeFile == 0) {
+			fclose(rcfile);
+			return false;
+		}
+		fseek(rcfile, 0, SEEK_SET);
+		std::vector<char> propsData(sizeFile);
+		int lenFile = static_cast<int>(fread(&propsData[0], 1, propsData.size(), rcfile));
 		fclose(rcfile);
-		const char *data = propsData;
+		const char *data = &propsData[0];
 		if (memcmp(data, "\xef\xbb\xbf", 3) == 0) {
 			data += 3;
 			lenFile -= 3;
@@ -579,6 +573,7 @@ SString PropSetFile::GetWild(const char *keybase, const char *filename) {
 // variable reference found.
 SString PropSetFile::GetNewExpand(const char *keybase, const char *filename) {
 	char *base = StringDup(GetWild(keybase, filename).c_str());
+	assert(base);
 	char *cpvar = strstr(base, "$(");
 	int maxExpands = 1000;	// Avoid infinite expansion of recursive definitions
 	while (cpvar && (maxExpands > 0)) {
@@ -586,6 +581,7 @@ SString PropSetFile::GetNewExpand(const char *keybase, const char *filename) {
 		if (cpendvar) {
 			ptrdiff_t lenvar = cpendvar - cpvar - 2;  	// Subtract the $()
 			char *var = StringDup(cpvar + 2, lenvar);
+			assert(var);
 			SString val = GetWild(var, filename);
 			if (0 == strcmp(var, keybase))
 				val.clear(); // Self-references evaluate to empty string
@@ -614,12 +610,6 @@ bool PropSetFile::GetFirst(const char *&key, const char *&val) {
 	if (it != props.end()) {
 		key = it->first.c_str();
 		val = it->second.c_str();
-		++it;
-		if (it != props.end()) {
-			enumnext = it->first; // GetNext will begin here ...
-		} else {
-			enumnext = "";
-		}
 		return true;
 	} else {
 		return false;
@@ -630,24 +620,16 @@ bool PropSetFile::GetFirst(const char *&key, const char *&val) {
  * Continue enumeration.
  */
 bool PropSetFile::GetNext(const char *&key, const char *&val) {
-	mapss::iterator it = props.find(enumnext);
+	mapss::iterator it = props.find(key);
 	if (it != props.end()) {
-		key = it->first.c_str();
-		val = it->second.c_str();
 		++it;
 		if (it != props.end()) {
-			enumnext = it->first; // GetNext will begin here ...
-		} else {
-			enumnext = "";
+			key = it->first.c_str();
+			val = it->second.c_str();
+			return true;
 		}
-		return true;
-	} else {
-		return false;
 	}
-}
-
-static inline bool IsLetter(char ch) {
-	return ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z'));
+	return false;
 }
 
 int CompareNoCase(const char *a, const char *b) {
@@ -809,6 +791,7 @@ SString &SString::append(const char *sOther, lenpos_t sLenOther, char sep) {
 			s[sLen] = sep;
 			sLen++;
 		}
+		assert(s);
 		memcpy(&s[sLen], sOther, sLenOther);
 		sLen += sLenOther;
 		s[sLen] = '\0';
