@@ -23,7 +23,6 @@ class PropSetFile {
 	SString GetWildUsingStart(const PropSetFile &psStart, const char *keybase, const char *filename);
 	static bool caseSensitiveFilenames;
 	mapss props;
-	std::string enumnext;
 public:
 	PropSetFile *superPS;
 	PropSetFile(bool lowerKeys_=false);
@@ -41,7 +40,6 @@ public:
 	SString Expand(const char *withVars, int maxExpands=100) const;
 	int GetInt(const char *key, int defaultValue=0) const;
 	void Clear();
-	char *ToString() const;	// Caller must delete[] the return value
 
 	bool ReadLine(const char *data, bool ifIsTrue, FilePath directoryForImports, const ImportFilter &filter, std::vector<FilePath> *imports=0);
 	void ReadFromMemory(const char *data, size_t len, FilePath directoryForImports, const ImportFilter &filter, std::vector<FilePath> *imports=0);
