@@ -93,11 +93,13 @@ class Buffer : public RecentFile {
 public:
 	sptr_t doc;
 	bool isDirty;
+	bool isReadOnly;
 	bool useMonoFont;
 	enum { empty, reading, readAll, open } lifeState;
 	UniMode unicodeMode;
 	time_t fileModTime;
 	time_t fileModLastAsk;
+	time_t documentModTime;
 	enum { fmNone, fmMarked, fmModified} findMarks;
 	SString overrideExtension;	///< User has chosen to use a particular language
 	std::vector<int> foldState;
@@ -106,17 +108,20 @@ public:
 	PropSetFile props;
 	enum FutureDo { fdNone=0, fdFinishSave=1 } futureDo;
 	Buffer() :
-			RecentFile(), doc(0), isDirty(false), useMonoFont(false), lifeState(empty),
-			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), findMarks(fmNone), pFileWorker(0), futureDo(fdNone) {}
+			RecentFile(), doc(0), isDirty(false), isReadOnly(false), useMonoFont(false), lifeState(empty),
+			unicodeMode(uni8Bit), fileModTime(0), fileModLastAsk(0), documentModTime(0),
+			findMarks(fmNone), pFileWorker(0), futureDo(fdNone) {}
 
 	void Init() {
 		RecentFile::Init();
 		isDirty = false;
+		isReadOnly = false;
 		useMonoFont = false;
 		lifeState = empty;
 		unicodeMode = uni8Bit;
 		fileModTime = 0;
 		fileModLastAsk = 0;
+		documentModTime = 0;
 		findMarks = fmNone;
 		overrideExtension = "";
 		foldState.clear();
@@ -128,10 +133,15 @@ public:
 	void SetTimeFromFile() {
 		fileModTime = ModifiedTime();
 		fileModLastAsk = fileModTime;
+		documentModTime = fileModTime;
 	}
 
+	void DocumentModified();
+	bool NeedsSave(int delayBeforeSave);
+
 	void CompleteLoading();
 	void CompleteStoring();
+	void AbandonAutomaticSave();
 
 	bool ShouldNotSave() const {
 		return lifeState != open;
@@ -177,6 +187,7 @@ public:
 	void MoveToStackTop(int index);
 	void ShiftTo(int indexFrom, int indexTo);
 	void Swap(int indexA, int indexB);
+	bool SingleBuffer() const;
 	BackgroundActivities CountBackgroundActivities() const;
 	bool SavingInBackground() const;
 	bool GetVisible(int index);
@@ -324,7 +335,8 @@ public:
 	virtual void MoveBack(int distance) = 0;
 	virtual void ScrollEditorIfNeeded() = 0;
 
-	virtual int FindNext(bool reverseDirection, bool showWarnings = true) = 0;
+	virtual int FindNext(bool reverseDirection, bool showWarnings = true, bool allowRegExp=true) = 0;
+	virtual void HideMatch() = 0;
 	virtual int MarkAll() = 0;
 	virtual int ReplaceAll(bool inSelection) = 0;
 	virtual void ReplaceOnce() = 0;
@@ -373,7 +385,7 @@ protected:
 	ComboMemory memFiles;
 	ComboMemory memDirectory;
 	SString parameterisedCommand;
-	char abbrevInsert[200];
+	SString abbrevInsert;
 
 	enum { languageCmdID = IDM_LANGUAGE };
 	LanguageMenuItem *languageMenu;
@@ -392,7 +404,8 @@ protected:
 	SString apisFileNames;
 	SString functionDefinition;
 
-	enum { diagnosticStyleStart=256, diagnosticStyleEnd=diagnosticStyleStart+4-1};
+	int diagnosticStyleStart;
+	enum { diagnosticStyles=4};
 
 	bool indentOpening;
 	bool indentClosing;
@@ -444,6 +457,10 @@ protected:
 	bool needReadProperties;
 	bool quitting;
 
+	int timerMask;
+	enum { timerAutoSave=1 };
+	int delayBeforeAutoSave;
+
 	int heightOutput;
 	int heightOutputStartDrag;
 	GUI::Point ptStartDrag;
@@ -460,8 +477,8 @@ protected:
 
 	bool indentationWSVisible;
 	int indentExamine;
-
 	bool autoCompleteIgnoreCase;
+	bool callTipUseEscapes;
 	bool callTipIgnoreCase;
 	bool autoCCausedByOnlyOne;
 	SString calltipWordCharacters;
@@ -554,6 +571,7 @@ protected:
 	void ReadDirectoryPropFile();
 
 	int CallFocused(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
+	int CallFocusedElseDefault(int defaultValue, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t CallPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	void CallChildren(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	SString GetTranslationToAbout(const char * const propname, bool retainIfNotFound = true);
@@ -624,13 +642,20 @@ protected:
 	int SaveIfUnsureAll(bool forceQuestion = false);
 	int SaveIfUnsureForBuilt();
 	bool SaveIfNotOpen(const FilePath &destFile, bool fixCase);
-	bool Save();
+	void AbandonAutomaticSave();
+	enum SaveFlags {
+	    sfNone = 0, 		// Default
+	    sfProgressVisible = 1, 	// Show in background save strip
+	    sfSynchronous = 16	// Write synchronously blocking UI
+	};
+	bool Save(SaveFlags sf = sfProgressVisible);
 	void SaveAs(const GUI::gui_char *file, bool fixCase);
 	virtual void SaveACopy() = 0;
 	void SaveToHTML(FilePath saveName);
 	void StripTrailingSpaces();
 	void EnsureFinalNewLine();
-	bool SaveBuffer(FilePath saveName, bool asynchronous);
+	bool PrepareBufferForSave(FilePath saveName);
+	bool SaveBuffer(FilePath saveName, SaveFlags sf);
 	virtual void SaveAsHTML() = 0;
 	void SaveToRTF(FilePath saveName, int start = 0, int end = -1);
 	virtual void SaveAsRTF() = 0;
@@ -652,12 +677,16 @@ protected:
 	int GetMenuCommandAsInt(SString commandName);
 	virtual void Print(bool) {}
 	virtual void PrintSetup() {}
+	virtual void UserStripShow(const char * /* description */) {}
+	virtual void UserStripSet(int /* control */, const char * /* value */) {}
+	virtual void UserStripSetList(int /* control */, const char * /* value */) {}
+	virtual const char *UserStripValue(int /* control */) { return 0; }
 	virtual void ShowBackgroundProgress(const GUI::gui_string & /* explanation */, int /* size */, int /* progress */) {}
 	Sci_CharacterRange GetSelection();
 	SelectedRange GetSelectedRange();
 	void SetSelection(int anchor, int currentPos);
 	//	void SelectionExtend(char *sel, int len, char *notselchar);
-	void GetCTag(char *sel, int len);
+	SString GetCTag();
 	SString GetRange(GUI::ScintillaWindow &win, int selStart, int selEnd);
 	virtual SString GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd);
 	SString GetLine(GUI::ScintillaWindow &win, int line);
@@ -680,7 +709,8 @@ protected:
 	virtual void SetReplace(const char *sReplace);
 	virtual void MoveBack(int distance);
 	virtual void ScrollEditorIfNeeded();
-	int FindNext(bool reverseDirection, bool showWarnings = true);
+	virtual int FindNext(bool reverseDirection, bool showWarnings = true, bool allowRegExp=true);
+	virtual void HideMatch();
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
 	virtual void FindInFiles() = 0;
@@ -704,9 +734,8 @@ protected:
 	void GoMatchingPreprocCond(int direction, bool select);
 	virtual void FindReplace(bool replace) = 0;
 	void OutputAppendString(const char *s, int len = -1);
-	void OutputAppendStringSynchronised(const char *s, int len = -1);
+	virtual void OutputAppendStringSynchronised(const char *s, int len = -1);
 	void MakeOutputVisible();
-	void ClearJobQueue();
 	virtual void Execute();
 	virtual void StopExecute() = 0;
 	void ShowMessages(int line);
@@ -720,6 +749,7 @@ protected:
 	virtual bool StartAutoComplete();
 	virtual bool StartAutoCompleteWord(bool onlyOneWord);
 	virtual bool StartExpandAbbreviation();
+	bool PerformInsertAbbreviation();
 	virtual bool StartInsertAbbreviation();
 	virtual bool StartBlockComment();
 	virtual bool StartBoxComment();
@@ -762,7 +792,7 @@ protected:
 	void FoldAll();
 	void ToggleFoldRecursive(int line, int level);
 	void EnsureAllChildrenVisible(int line, int level);
-	void EnsureRangeVisible(int posStart, int posEnd, bool enforcePolicy = true);
+	void EnsureRangeVisible(GUI::ScintillaWindow &win, int posStart, int posEnd, bool enforcePolicy = true);
 	void GotoLineEnsureVisible(int line);
 	bool MarginClick(int position, int modifiers);
 	void NewLineInOutput();
@@ -780,7 +810,7 @@ protected:
 	bool BookmarkPresent(int lineno = -1);
 	void BookmarkToggle(int lineno = -1);
 	void BookmarkNext(bool forwardScan = true, bool select = false);
-	void ToggleOutputVisible();
+	virtual void ToggleOutputVisible();
 	virtual void SizeContentWindows() = 0;
 	virtual void SizeSubWindows() = 0;
 
@@ -849,6 +879,11 @@ protected:
 	int NormaliseSplit(int splitPos);
 	void MoveSplit(GUI::Point ptNewDrag);
 
+	virtual void TimerStart(int mask);
+	virtual void TimerEnd(int mask);
+	void OnTimer();
+
+	void SetHomeProperties();
 	void UIAvailable();
 	void PerformOne(char *action);
 	void StartRecordMacro();
@@ -869,7 +904,8 @@ protected:
 	};
 	virtual bool GrepIntoDirectory(const FilePath &directory);
 	void GrepRecursive(GrepFlags gf, FilePath baseDir, const char *searchString, const GUI::gui_char *fileTypes);
-	void InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *files, const char *search);
+	void InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *files,
+			  const char *search, sptr_t &originalEnd);
 	void EnumProperties(const char *action);
 	void SendOneProperty(const char *kind, const char *key, const char *val);
 	void PropertyFromDirector(const char *arg);
@@ -930,7 +966,7 @@ private:
 #endif
 
 int ControlIDOfCommand(unsigned long);
-void LowerCaseString(char *s);
+std::vector<GUI::gui_string> ListFromString(const GUI::gui_string &args);
 long ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault);
 void WindowSetFocus(GUI::ScintillaWindow &w);
 
