@@ -360,24 +360,29 @@ void SciTEBase::TextRead(FileWorker *pFileWorker) {
 void SciTEBase::PerformDeferredTasks() {
 	if (buffers.buffers[buffers.Current()].futureDo & Buffer::fdFinishSave) {
 		wEditor.Call(SCI_SETSAVEPOINT);
-		wEditor.Call(SCI_SETREADONLY, isReadOnly);
+		wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
 		buffers.FinishedFuture(buffers.Current(), Buffer::fdFinishSave);
 	}
 }
 
 void SciTEBase::CompleteOpen(OpenCompletion oc) {
 	wEditor.Call(SCI_SETREADONLY, isReadOnly);
+
+	if (oc != ocSynchronous) {
+		ReadProperties();
+	}
+
 	if (language == "") {
 		SString languageOverride = DiscoverLanguage();
 		if (languageOverride.length()) {
 			CurrentBuffer()->overrideExtension = languageOverride;
+			CurrentBuffer()->lifeState = Buffer::open;
 			ReadProperties();
 			SetIndentSettings();
 		}
 	}
 
 	if (oc != ocSynchronous) {
-		ReadProperties();
 		SetIndentSettings();
 		SetEol();
 		UpdateBuffersCurrent();
@@ -418,20 +423,19 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 
 	FilePath pathSaved = pFileStorer->path;
 	int errSaved = pFileStorer->err;
+	bool cancelledSaved = pFileStorer->cancelling;
 
 	// May not be found if save cancelled or buffer closed
 	if (iBuffer >= 0) {
 		// Complete and release
-		//pFileStorer->pStorer->SaveCompleted();
-		//pFileStorer->pStorer = 0;
 		buffers.buffers[iBuffer].CompleteStoring();
-		if (errSaved) {
+		if (errSaved || cancelledSaved) {
 			// Background save failed (possibly out-of-space) so resurrect the 
 			// buffer so can be saved to another disk or retried after making room.
 			buffers.SetVisible(iBuffer, true);
 			SetBuffersMenu();
 			if (iBuffer == buffers.Current()) {
-				wEditor.Call(SCI_SETREADONLY, isReadOnly);
+				wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
 			}
 		} else {
 			if (!buffers.GetVisible(iBuffer)) {
@@ -439,12 +443,14 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 			}
 			if (iBuffer == buffers.Current()) {
 				wEditor.Call(SCI_SETSAVEPOINT);
-				wEditor.Call(SCI_SETREADONLY, isReadOnly);
+				wEditor.Call(SCI_SETREADONLY, CurrentBuffer()->isReadOnly);
 				if (extender)
 					extender->OnSave(buffers.buffers[iBuffer].AsUTF8().c_str());
 			} else {
+				buffers.buffers[iBuffer].isDirty = false;
 				// Need to make writable and set save point when next receive focus.
 				buffers.AddFuture(iBuffer, Buffer::fdFinishSave);
+				SetBuffersMenu();
 			}
 		}
 	} else {
@@ -461,7 +467,7 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 		ReloadProperties();
 	}
 	UpdateStatusBar(true);
-	if (!jobQueue.executing && (jobQueue.commandCurrent > 0)) {
+	if (!jobQueue.executing && (jobQueue.HasCommandToRun())) {
 		Execute();
 	}
 	if (quitting && !buffers.SavingInBackground()) {
@@ -515,7 +521,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		return false;
 	}
 
-	long size = absPath.GetFileLength();
+	long size = absPath.IsUntitled() ? 0 : absPath.GetFileLength();
 	if (size > 0) {
 		// Real file, not empty buffer
 		int maxSize = props.GetInt("max.file.size");
@@ -585,6 +591,7 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 			wEditor.Call(SCI_EMPTYUNDOBUFFER);
 		}
 		isReadOnly = props.GetInt("read.only");
+		CurrentBuffer()->isReadOnly = isReadOnly;
 		wEditor.Call(SCI_SETREADONLY, isReadOnly);
 	}
 	RemoveFileFromStack(filePath);
@@ -601,77 +608,79 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 
 // Returns true if editor should get the focus
 bool SciTEBase::OpenSelected() {
-	char selectedFilename[MAX_PATH];
-	char cTag[200];
-	unsigned long lineNumber = 0;
-
 	SString selName = SelectionFilename();
-	strncpy(selectedFilename, selName.c_str(), MAX_PATH);
-	selectedFilename[MAX_PATH - 1] = '\0';
-	if (selectedFilename[0] == '\0') {
+	if (selName.length() == 0) {
 		WarnUser(warnWrongFile);
 		return false;	// No selection
 	}
+
+#if !defined(GTK)
+	if (selName.startswith("http:") ||
+	        selName.startswith("https:") ||
+	        selName.startswith("ftp:") ||
+	        selName.startswith("ftps:") ||
+	        selName.startswith("news:") ||
+	        selName.startswith("mailto:")) {
+		SString cmd = selName;
+		AddCommand(cmd, "", jobShell);
+		return false;	// Job is done
+	}
+#endif
+
+	if (selName.startswith("file://")) {
+		selName.remove(0, 7);
+		if (selName[0] == '/' && selName[2] == ':') { // file:///C:/filename.ext
+			selName.remove(0, 1);
+		}
+	}
+
 	SString fileNameForExtension = ExtensionFileName();
 	SString openSuffix = props.GetNewExpand("open.suffix.", fileNameForExtension.c_str());
-	strcat(selectedFilename, openSuffix.c_str());
+	selName += openSuffix;
 
-	if (EqualCaseInsensitive(selectedFilename, FileNameExt().AsUTF8().c_str()) || EqualCaseInsensitive(selectedFilename, filePath.AsUTF8().c_str())) {
+	if (EqualCaseInsensitive(selName.c_str(), FileNameExt().AsUTF8().c_str()) || EqualCaseInsensitive(selName.c_str(), filePath.AsUTF8().c_str())) {
 		WarnUser(warnWrongFile);
 		return true;	// Do not open if it is the current file!
 	}
 
-	cTag[0] = '\0';
+	SString cTag;
+	unsigned long lineNumber = 0;
 	if (IsPropertiesFile(filePath) &&
-	        strchr(selectedFilename, '.') == 0 &&
-	        strlen(selectedFilename) + strlen(PROPERTIES_EXTENSION) < MAX_PATH) {
+	        !selName.contains('.')) {
 		// We are in a properties file and try to open a file without extension,
 		// we suppose we want to open an imported .properties file
 		// So we append the correct extension to open the included file.
 		// Maybe we should check if the filename is preceded by "import"...
-		strcat(selectedFilename, PROPERTIES_EXTENSION);
+		selName += PROPERTIES_EXTENSION;
 	} else {
 		// Check if we have a line number (error message or grep result)
 		// A bit of duplicate work with DecodeMessage, but we don't know
 		// here the format of the line, so we do guess work.
 		// Can't do much for space separated line numbers anyway...
-		char *endPath = strchr(selectedFilename, '(');
-		if (endPath) {	// Visual Studio error message: F:\scite\src\SciTEBase.h(312):	bool Exists(
-			lineNumber = atol(endPath + 1);
+		int endPath = selName.search("(");
+		if (endPath >= 0) {	// Visual Studio error message: F:\scite\src\SciTEBase.h(312):	bool Exists(
+			lineNumber = atol(selName.c_str() + endPath + 1);
 		} else {
-			endPath = strchr(selectedFilename + 2, ':');	// Skip Windows' drive separator
-			if (endPath) {	// grep -n line, perhaps gcc too: F:\scite\src\SciTEBase.h:312:	bool Exists(
-				lineNumber = atol(endPath + 1);
+			endPath = selName.search(":", 2);	// Skip Windows' drive separator
+			if (endPath >= 0) {	// grep -n line, perhaps gcc too: F:\scite\src\SciTEBase.h:312:	bool Exists(
+				lineNumber = atol(selName.c_str() + endPath + 1);
 			}
 		}
 		if (lineNumber > 0) {
-			*endPath = '\0';
+			selName.remove(endPath, 0);
 		}
 
-#if !defined(GTK)
-		if (strncmp(selectedFilename, "http:", 5) == 0 ||
-		        strncmp(selectedFilename, "https:", 6) == 0 ||
-		        strncmp(selectedFilename, "ftp:", 4) == 0 ||
-		        strncmp(selectedFilename, "ftps:", 5) == 0 ||
-		        strncmp(selectedFilename, "news:", 5) == 0 ||
-		        strncmp(selectedFilename, "mailto:", 7) == 0) {
-			SString cmd = selectedFilename;
-			AddCommand(cmd, "", jobShell);
-			return false;	// Job is done
-		}
-#endif
-
 		// Support the ctags format
 
 		if (lineNumber == 0) {
-			GetCTag(cTag, sizeof(cTag));
+			cTag = GetCTag();
 		}
 	}
 
 	FilePath path;
 	// Don't load the path of the current file if the selected
 	// filename is an absolute pathname
-	GUI::gui_string selFN = GUI::StringFromUTF8(selectedFilename);
+	GUI::gui_string selFN = GUI::StringFromUTF8(selName.c_str());
 	if (!FilePath(selFN).IsAbsolute()) {
 		path = filePath.Directory();
 		// If not there, look in openpath
@@ -696,12 +705,14 @@ bool SciTEBase::OpenSelected() {
 	}
 	FilePath pathReturned;
 	if (Exists(path.AsInternal(), selFN.c_str(), &pathReturned)) {
-		if (Open(pathReturned)) {
+		// Open synchronously if want to seek line number or search tag
+		OpenFlags of = ((lineNumber > 0) || (cTag.length() != 0)) ? ofSynchronous : ofNone;
+		if (Open(pathReturned, of)) {
 			if (lineNumber > 0) {
 				wEditor.Call(SCI_GOTOLINE, lineNumber - 1);
-			} else if (cTag[0] != '\0') {
-				if (atol(cTag) > 0) {
-					wEditor.Call(SCI_GOTOLINE, atol(cTag) - 1);
+			} else if (cTag.length() != 0) {
+				if (cTag.value() > 0) {
+					wEditor.Call(SCI_GOTOLINE, cTag.value() - 1);
 				} else {
 					findWhat = cTag;
 					FindNext(false);
@@ -787,8 +798,12 @@ FilePath SciTEBase::SaveName(const char *ext) {
 }
 
 int SciTEBase::SaveIfUnsure(bool forceQuestion) {
-	if (CurrentBuffer()->pFileWorker && !CurrentBuffer()->pFileWorker->IsLoading()) {
-		return IDNO;
+	if (CurrentBuffer()->pFileWorker) {
+		if (CurrentBuffer()->pFileWorker->IsLoading())
+			// In semi-loaded state so refuse to save
+			return IDCANCEL;
+		else
+			return IDNO;
 	}
 	if ((CurrentBuffer()->isDirty) && (LengthDocument() || !filePath.IsUntitled() || forceQuestion)) {
 		if (props.GetInt("are.you.sure", 1) ||
@@ -905,14 +920,16 @@ void SciTEBase::EnsureFinalNewLine() {
 	}
 }
 
-/**
- * Writes the buffer to the given filename.
- */
-bool SciTEBase::SaveBuffer(FilePath saveName, bool asynchronous) {
+// Perform any changes needed before saving such as normalizing spaces and line ends.
+bool SciTEBase::PrepareBufferForSave(FilePath saveName) {
 	bool retVal = false;
 	// Perform clean ups on text before saving
 	wEditor.Call(SCI_BEGINUNDOACTION);
-	if (props.GetInt("strip.trailing.spaces"))
+	SString useStripTrailingSpaces = props.GetNewExpand("strip.trailing.spaces.", ExtensionFileName().c_str());
+	if (useStripTrailingSpaces.length() > 0) {
+		if (useStripTrailingSpaces.value())
+			StripTrailingSpaces();
+	} else if (props.GetInt("strip.trailing.spaces"))
 		StripTrailingSpaces();
 	if (props.GetInt("ensure.final.line.end"))
 		EnsureFinalNewLine();
@@ -924,18 +941,31 @@ bool SciTEBase::SaveBuffer(FilePath saveName, bool asynchronous) {
 
 	wEditor.Call(SCI_ENDUNDOACTION);
 
+	return retVal;
+}
+
+/**
+ * Writes the buffer to the given filename.
+ */
+bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
+	bool retVal = PrepareBufferForSave(saveName);
+
 	if (!retVal) {
 
 		FILE *fp = saveName.Open(fileWrite);
 		if (fp) {
 			int lengthDoc = LengthDocument();
-			if (asynchronous) {
+			if (!(sf & sfSynchronous)) {
 				wEditor.Call(SCI_SETREADONLY, 1);
 				const char *documentBytes = reinterpret_cast<const char *>(wEditor.CallReturnPointer(SCI_GETCHARACTERPOINTER));
-				CurrentBuffer()->pFileWorker = new FileStorer(this, documentBytes, filePath, lengthDoc, fp, CurrentBuffer()->unicodeMode);
+				CurrentBuffer()->pFileWorker = new FileStorer(this, documentBytes, filePath, lengthDoc, fp, CurrentBuffer()->unicodeMode, (sf & sfProgressVisible));
 				CurrentBuffer()->pFileWorker->sleepTime = props.GetInt("asynchronous.sleep");
-				PerformOnNewThread(CurrentBuffer()->pFileWorker);
-				retVal = true;
+				if (PerformOnNewThread(CurrentBuffer()->pFileWorker)) {
+					retVal = true;
+				} else {
+					GUI::gui_string msg = LocaliseMessage("Failed to save file '^0' as thread could not be started.", filePath.AsInternal());
+					WindowMessageBox(wSciTE, msg, MB_OK | MB_ICONWARNING);
+				}
 			} else {
 				Utf8_16_Write convert;
 				if (CurrentBuffer()->unicodeMode != uniCookie) {	// Save file with cookie without BOM.
@@ -983,7 +1013,7 @@ void SciTEBase::ReloadProperties() {
 }
 
 // Returns false if cancelled or failed to save
-bool SciTEBase::Save() {
+bool SciTEBase::Save(SaveFlags sf) {
 	if (!filePath.IsUntitled()) {
 		GUI::gui_string msg;
 		if (CurrentBuffer()->ShouldNotSave()) {
@@ -1021,10 +1051,12 @@ bool SciTEBase::Save() {
 			}
 		}
 
-		bool asynchronous = LengthDocument() > props.GetInt("background.save.size", -1);
-		if (SaveBuffer(filePath, asynchronous)) {
+		if ((LengthDocument() <= props.GetInt("background.save.size", -1)) ||
+			(buffers.SingleBuffer()))
+			sf = static_cast<SaveFlags>(sf | sfSynchronous);
+		if (SaveBuffer(filePath, sf)) {
 			CurrentBuffer()->SetTimeFromFile();
-			if (!asynchronous) {
+			if (sf & sfSynchronous) {
 				wEditor.Call(SCI_SETSAVEPOINT);
 				if (IsPropertiesFile(filePath)) {
 					ReloadProperties();
@@ -1050,7 +1082,7 @@ void SciTEBase::SaveAs(const GUI::gui_char *file, bool fixCase) {
 	Save();
 	ReadProperties();
 	wEditor.Call(SCI_CLEARDOCUMENTSTYLE);
-	wEditor.Call(SCI_COLOURISE, 0, -1);
+	wEditor.Call(SCI_COLOURISE, 0, wEditor.Call(SCI_POSITIONFROMLINE, 1));
 	Redraw();
 	SetWindowName();
 	BuffersMenu();
@@ -1072,6 +1104,10 @@ bool SciTEBase::SaveIfNotOpen(const FilePath &destFile, bool fixCase) {
 	}
 }
 
+void SciTEBase::AbandonAutomaticSave() {
+	CurrentBuffer()->AbandonAutomaticSave();
+}
+
 bool SciTEBase::IsStdinBlocked() {
 	return false; /* always default to blocked */
 }
@@ -1144,13 +1180,13 @@ void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 
 void SciTEBase::OpenFilesFromStdin() {
 	char data[blockSize];
-	char *pNL;
 
 	/* if stdin is blocked, do not execute this method */
 	if (IsStdinBlocked())
 		return;
 
 	while (fgets(data, sizeof(data) - 1, stdin)) {
+		char *pNL;
 		if ((pNL = strchr(data, '\n')) != NULL)
 			* pNL = '\0';
 		Open(GUI::StringFromUTF8(data).c_str(), ofQuiet);
@@ -1222,6 +1258,8 @@ class FileReader {
 	char lineToCompare[bufLen+1];
 	char lineToShow[bufLen+1];
 	bool caseSensitive;
+	// Private so FileReader objects can not be copied
+	FileReader(const FileReader &);
 public:
 
 	FileReader(FilePath fPath, bool caseSensitive_) {
@@ -1243,7 +1281,6 @@ public:
 			int ch = bf->NextByte();
 			if (i == 0 && lastWasCR && ch == '\n') {
 				lastWasCR = false;
-				ch = 0;
 			} else if (ch == '\r' || ch == '\n') {
 				lastWasCR = ch == '\r';
 				break;
@@ -1297,7 +1334,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *search
 		if (jobQueue.Cancelled())
 			return;
 		FilePath fPath = files[i];
-		if (fPath.Matches(fileTypes)) {
+		if (*fileTypes == '\0' || fPath.Matches(fileTypes)) {
 			//OutputAppendStringSynchronised(i->AsInternal());
 			//OutputAppendStringSynchronised("\n");
 			FileReader fr(fPath, gf & grepMatchCase);
@@ -1344,8 +1381,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *search
 	}
 }
 
-void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes, const char *search) {
-	sptr_t originalEnd = 0;
+void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes, const char *search, sptr_t &originalEnd) {
 	GUI::ElapsedTime commandTime;
 	if (!(gf & grepStdOut)) {
 		SString os;
@@ -1356,7 +1392,7 @@ void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const
 		os.append("\"\n");
 		OutputAppendStringSynchronised(os.c_str());
 		MakeOutputVisible();
-		originalEnd = wOutput.Send(SCI_GETCURRENTPOS);
+		originalEnd += os.length();
 	}
 	SString searchString(search);
 	if (!(gf & grepMatchCase)) {
@@ -1371,8 +1407,6 @@ void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const
 		}
 		sExitMessage += "\n";
 		OutputAppendStringSynchronised(sExitMessage.c_str());
-		if ((gf & grepScroll) && returnOutputToCommand)
-			wOutput.Send(SCI_GOTOPOS, originalEnd, 0);
 	}
 }
 
