@@ -453,15 +453,6 @@ void SciTEBase::SetStyleFor(GUI::ScintillaWindow &win, const char *lang) {
 	SetStyleBlock(win, lang, 0, maxStyle);
 }
 
-void LowerCaseString(char *s) {
-	while (*s) {
-		if ((*s >= 'A') && (*s <= 'Z')) {
-			*s = static_cast<char>(*s - 'A' + 'a');
-		}
-		s++;
-	}
-}
-
 SString SciTEBase::ExtensionFileName() {
 	if (CurrentBuffer()->overrideExtension.length()) {
 		return CurrentBuffer()->overrideExtension;
@@ -469,15 +460,14 @@ SString SciTEBase::ExtensionFileName() {
 		FilePath name = FileNameExt();
 		if (name.IsSet()) {
 			// Force extension to lower case
-			char fileNameWithLowerCaseExtension[MAX_PATH];
-			strcpy(fileNameWithLowerCaseExtension, name.AsUTF8().c_str());
+			SString fileNameWithLowerCaseExtension = name.AsUTF8().c_str();
 #if !defined(GTK)
-			char *extension = strrchr(fileNameWithLowerCaseExtension, '.');
+			const char *extension = strrchr(fileNameWithLowerCaseExtension.c_str(), '.');
 			if (extension) {
-				LowerCaseString(extension);
+				fileNameWithLowerCaseExtension.lowercase(extension - fileNameWithLowerCaseExtension.c_str());
 			}
 #endif
-			return SString(fileNameWithLowerCaseExtension);
+			return fileNameWithLowerCaseExtension;
 		} else {
 			return props.Get("default.file.ext");
 		}
@@ -628,9 +618,13 @@ static const char *propertiesToForward[] = {
 	"lexer.cpp.track.preprocessor",
 	"lexer.cpp.triplequoted.strings",
 	"lexer.cpp.update.preprocessor",
+	"lexer.css.hss.language",
+	"lexer.css.less.language",
+	"lexer.css.scss.language",
 	"lexer.d.fold.at.else",
 	"lexer.errorlist.value.separate",
 	"lexer.flagship.styling.within.preprocessor",
+	"lexer.haskell.allow.hash",
 	"lexer.html.django",
 	"lexer.html.mako",
 	"lexer.metapost.comment.process",
@@ -1010,10 +1004,11 @@ void SciTEBase::ReadProperties() {
 
 	sval = FindLanguageProperty("calltip.*.ignorecase");
 	callTipIgnoreCase = sval == "1";
+	sval = FindLanguageProperty("calltip.*.use.escapes");
+	callTipUseEscapes = sval == "1";
 
 	calltipWordCharacters = FindLanguageProperty("calltip.*.word.characters",
 		"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
-
 	calltipParametersStart = FindLanguageProperty("calltip.*.parameters.start", "(");
 	calltipParametersEnd = FindLanguageProperty("calltip.*.parameters.end", ")");
 	calltipParametersSeparators = FindLanguageProperty("calltip.*.parameters.separators", ",;");
@@ -1045,7 +1040,7 @@ void SciTEBase::ReadProperties() {
 	wOutput.Call(SCI_AUTOCSETIGNORECASE, 1);
 
 	int autoCChooseSingle = props.GetInt("autocomplete.choose.single");
-	wEditor.Call(SCI_AUTOCSETCHOOSESINGLE, autoCChooseSingle),
+	wEditor.Call(SCI_AUTOCSETCHOOSESINGLE, autoCChooseSingle);
 
 	wEditor.Call(SCI_AUTOCSETCANCELATSTART, 0);
 	wEditor.Call(SCI_AUTOCSETDROPRESTOFWORD, 0);
@@ -1376,6 +1371,14 @@ void SciTEBase::ReadProperties() {
 			}
 		}
 	}
+
+	delayBeforeAutoSave = props.GetInt("save.on.timer");
+	if (delayBeforeAutoSave) {
+		TimerStart(timerAutoSave);
+	} else {
+		TimerEnd(timerAutoSave);
+	}
+
 	firstPropertiesRead = false;
 	needReadProperties = false;
 }
@@ -1416,8 +1419,9 @@ void SciTEBase::ReadFontProperties() {
 	SetStyleFor(wEditor, "*");
 	SetStyleFor(wEditor, languageName);
 	if (props.GetInt("error.inline")) {
-		wEditor.Send(SCI_STYLESETFORE, diagnosticStyleEnd, 0);	// Ensure styles allocated
-		SetStyleBlock(wEditor, "error", diagnosticStyleStart, diagnosticStyleEnd);
+		wEditor.Call(SCI_RELEASEALLEXTENDEDSTYLES, 0, 0);
+		diagnosticStyleStart = wEditor.Call(SCI_ALLOCATEEXTENDEDSTYLES, diagnosticStyles, 0);
+		SetStyleBlock(wEditor, "error", diagnosticStyleStart, diagnosticStyleStart+diagnosticStyles-1);
 	}
 
 	// Turn grey while loading
